<?php
/**
*
* @package phpBB Extension - Hangman Game
* @copyright (c) 2017 dmzx - http://www.dmzx-web.net
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace dmzx\hangmangame\controller;

class admin_controller
{
	/** @var \dmzx\hangmangame\core\functions_hangmangame */
	protected $functions;

	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\user */
	protected $user;

	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\controller\helper */
	protected $helper;

	/** @var \phpbb\log\log_interface */
	protected $log;

	/**
	* The database tables
	*
	* @var string
	*/
	protected $hangmangame_table_hangman_quess;

	protected $hangmangame_table_hangman_config;

	protected $hangmangame_table_hangman_words;

	protected $hangmangame_table_hangman_score;

	/**
	* Constructor
	*
	* @param \dmzx\hangmangame\core\functions_hangmangame	$functions
	* @param \phpbb\template\template		 				$template
	* @param \phpbb\user									$user
	* @param \phpbb\auth\auth								$auth
	* @param \phpbb\db\driver\driver_interface				$db
	* @param \phpbb\request\request		 					$request
	* @param \phpbb\config\config							$config
	* @param \phpbb\controller\helper		 				$helper
	* @param \\phpbb\log\log_interface						$log
	* @param string 										$hangmangame_table_hangman_quess
	* @param string 										$hangmangame_table_hangman_config
	* @param string 										$hangmangame_table_hangman_words
	* @param string 										$hangmangame_table_hangman_score
	*
	*/
	public function __construct(
		\dmzx\hangmangame\core\functions_hangmangame $functions,
		\phpbb\template\template $template,
		\phpbb\user $user,
		\phpbb\auth\auth $auth,
		\phpbb\db\driver\driver_interface $db,
		\phpbb\request\request $request,
		\phpbb\config\config $config,
		\phpbb\controller\helper $helper,
		\phpbb\log\log_interface $log,
		$hangmangame_table_hangman_quess,
		$hangmangame_table_hangman_config,
		$hangmangame_table_hangman_words,
		$hangmangame_table_hangman_score

	)
	{
		$this->functions 						= $functions;
		$this->template 						= $template;
		$this->user 							= $user;
		$this->auth 							= $auth;
		$this->db 								= $db;
		$this->request 							= $request;
		$this->config 							= $config;
		$this->helper 							= $helper;
		$this->log 								= $log;
		$this->hangmangame_table_hangman_quess	= $hangmangame_table_hangman_quess;
		$this->hangmangame_table_hangman_config = $hangmangame_table_hangman_config;
		$this->hangmangame_table_hangman_words 	= $hangmangame_table_hangman_words;
		$this->hangmangame_table_hangman_score 	= $hangmangame_table_hangman_score;
	}

	public function handle_settings()
	{
		$this->user->add_lang_ext('dmzx/hangmangame', 'acp_hangmangame');

		$hangman_config = $this->functions->hangman_config();
		$submit = $this->request->is_set_post('submit');

		// Form key
		add_form_key('acp_hangman');

		$this->template->assign_vars(array(
			'BASE'						=> $this->u_action,
			'HANGMANGAME_VERSION'		=> $this->config['hangmangame_version'],
		));

		//submitted vars
		$action				= $this->request->variable('action', '');
		$del_hangmans 		= $this->request->variable('del_hangmans', '');
		$del_highscore 		= $this->request->variable('del_highscore', '');
		$del_won			= $this->request->variable('del_won', '');
		$del_time_up		= $this->request->variable('del_time_up', '');
		$del_created		= $this->request->variable('del_created', '');
		$default_config 	= $hangman_config;

		if ($del_hangmans)
		{
			if (confirm_box(true))
			{
				if (!$this->auth->acl_get('a_hangman_set'))
				{
					trigger_error($this->user->lang['NO_AUTH_OPERATION'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				$sql = 'TRUNCATE ' . $this->hangmangame_table_hangman_quess;
				$this->db->sql_query($sql);
				$sql = 'TRUNCATE ' . $this->hangmangame_table_hangman_words;
				$this->db->sql_query($sql);
				$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'LOG_DELETED_ALL_HANGMANS');
				trigger_error($this->user->lang['ACP_DELETED_ALL_HANGMANS'] . adm_back_link($this->u_action));
			}
			else
			{
				$s_hidden_fields = array(
					'action'		=> $action,
					'del_hangmans'	=> true,
				);
				confirm_box(false, $this->user->lang['ACP_DELETE_ALL_HANGMANS_CONFIRM_MESSAGE'], build_hidden_fields($s_hidden_fields));
			}
		}

		if ($del_highscore)
		{
			if (confirm_box(true))
			{
				if (!$this->auth->acl_get('a_hangman_set'))
				{
					trigger_error($this->user->lang['NO_AUTH_OPERATION'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				$sql = 'TRUNCATE ' . $this->hangmangame_table_hangman_score;
				$this->db->sql_query($sql);
				//prevent negative scores ;)
				$sql = 'UPDATE ' . $this->hangmangame_table_hangman_words . " SET user_id = 0";
				$this->db->sql_query($sql);
				$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'LOG_DELETED_HIGHSCORE');
				trigger_error($this->user->lang['ACP_DELETED_HIGHSCORE'] . adm_back_link($this->u_action));
			}
			else
			{
				$s_hidden_fields = array(
					'action'		=> $action,
					'del_highscore'	=> true,
				);
				confirm_box(false, $this->user->lang['ACP_DELETE_HIGHSCORE_CONFIRM_MESSAGE'], build_hidden_fields($s_hidden_fields));
			}
		}

		if ($del_won)
		{
			if (confirm_box(true))
			{
				if (!$this->auth->acl_get('a_hangman_set'))
				{
					trigger_error($this->user->lang['NO_AUTH_OPERATION'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				$sql = 'DELETE FROM ' . $this->hangmangame_table_hangman_words . ' WHERE bwon = 1';
				$result = $this->db->sql_query($sql);
				$deleted = $this->db->sql_affectedrows();
				$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'LOG_DELETED_WON_HANGMANS');
				trigger_error(sprintf($this->user->lang['ACP_DELETED_HANGMANS'], $deleted ) . adm_back_link($this->u_action));
			}
			else
			{
				$s_hidden_fields = array(
					'action'		=> $action,
					'del_won'		=> true,
				);
				confirm_box(false, $this->user->lang['ACP_DELETE_WON_CONFIRM_MESSAGE'], build_hidden_fields($s_hidden_fields));
			}
		}

		if ($del_time_up)
		{
			if (confirm_box(true))
			{
				if (!$this->auth->acl_get('a_hangman_set'))
				{
					trigger_error($this->user->lang['NO_AUTH_OPERATION'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				$sql = 'DELETE FROM ' . $this->hangmangame_table_hangman_words . ' WHERE time_limit < ' . time() . ' AND time_limit <> 0';
				$this->db->sql_query($sql);
				$deleted = $this->db->sql_affectedrows();
				$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'LOG_DELETED_EXP_HANGMANS');
				trigger_error(sprintf($this->user->lang['ACP_DELETED_HANGMANS'], $deleted ) . adm_back_link($this->u_action));
			}
			else
			{
				$s_hidden_fields = array(
					'action'		=> $action,
					'del_time_up'	=> true,
				);
				confirm_box(false, $this->user->lang['ACP_DELETE_EXP_CONFIRM_MESSAGE'], build_hidden_fields($s_hidden_fields));
			}
		}

		if ($del_created)
		{
			if (confirm_box(true))
			{
				if (!$this->auth->acl_get('a_hangman_set'))
				{
					trigger_error($this->user->lang['NO_AUTH_OPERATION'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				$sql = 'UPDATE ' . $this->hangmangame_table_hangman_config . " SET config_value = '0' WHERE config_name LIKE 'hangman_counter'";
				$this->db->sql_query($sql);
				$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'LOG_RESET_CREATED_HANGMANS');
				trigger_error($this->user->lang['ACP_RESET_CREATED_HANGMANS'] . adm_back_link($this->u_action));
			}
			else
			{
				$s_hidden_fields = array(
					'action'		=> $action,
					'del_created'	=> true,
				);
				confirm_box(false, $this->user->lang['ACP_RESET_CREATED_CONFIRM_MESSAGE'], build_hidden_fields($s_hidden_fields));
			}
		}
		$sql = 'SELECT *
			FROM ' . $this->hangmangame_table_hangman_config;
		$result = $this->db->sql_query($sql);
		$rows = $this->db->sql_fetchrowset($result);
		$this->db->sql_freeresult($result);
		$rowcount = sizeof($rows);
		$default_config = array(); //Grab new acting hangman config
		$sql = 'SELECT *
			FROM ' . $this->hangmangame_table_hangman_config;
		$result = $this->db->sql_query($sql);
		while ($row = $this->db->sql_fetchrow($result))
		{
			$default_config[$row['config_name']] = $row['config_value'];
		}
		$this->db->sql_freeresult($result);

		$new = $default_config;

		if ($submit)
		{
			// Checking entered values for config
			// games per page
			if ($this->request->variable('games_per_page', 0) < 1)
			{
				trigger_error($this->user->lang['ACP_ERROR_GAMES_PER_PAGE'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			// time play again
			if ($this->request->variable('time_play_again', 0) < 0)
			{
				trigger_error($this->user->lang['ACP_ERROR_TIME_PLAY'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			// standard tries
			if ($this->request->variable('standart_tries', 0) < 1)
			{
				trigger_error($this->user->lang['ACP_ERROR_STD_TRIES'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			// min help letters
			if ($this->request->variable('min_help', 0) < 0)
			{
				trigger_error($this->user->lang['ACP_ERROR_MIN_HELP'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			// min tries
			if ($this->request->variable('minimum_tries', 0) < 1 || $this->request->variable('minimum_tries', 0) > 20)
			{
				trigger_error($this->user->lang['ACP_ERROR_MIN_TRIES'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			// max tries
			if ($this->request->variable('maximum_tries', 0) < 1 || $this->request->variable('maximum_tries', 0) > 20)
			{
				trigger_error($this->user->lang['ACP_ERROR_MAX_TRIES'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			// min > max tries
			if ($this->request->variable('minimum_tries', 0) > $this->request->variable('maximum_tries', 0))
			{
				trigger_error($this->user->lang['ACP_ERROR_MIN_MAX'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			// min letters
			if ($this->request->variable('minimum_letters', 0) < 1)
			{
				trigger_error($this->user->lang['ACP_ERROR_MIN_LETTERS'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			// max letters
			if ($this->request->variable('maximum_letters', 0) < -1)
			{
				trigger_error($this->user->lang['ACP_ERROR_MAX_LETTERS'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			// max < min letters
			if (($this->request->variable('maximum_letters', 0) < $this->request->variable('minimum_letters', 0)) && ($this->request->variable('maximum_letters', 0) != -1))
			{
				trigger_error($this->user->lang['ACP_ERROR_MIN_MAX_LET'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
		}

		foreach ($rows as $row)
		{
			$config_name = $row['config_name'];
			$config_value = $row['config_value'];
			$default_config[$config_name] = $config_value;
			$new[$config_name] = $this->request->variable($config_name, $default_config[$config_name], true);
			if ($submit)
			{
				$sql = 'UPDATE ' . $this->hangmangame_table_hangman_config . "
					SET config_value = '" . $this->db->sql_escape($new[$config_name]) . "'
					WHERE config_name = '$config_name'";
				$this->db->sql_query($sql);
			}
		}

		if ($submit)
		{
			if (!check_form_key('acp_hangman'))
			{
				trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			$this->config->set('hangman_enable', $this->request->variable('hangman_enable', ''), true);
			$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'LOG_CHANGED_SETTINGS');
			trigger_error($this->user->lang['ACP_CONFIG_UPDATED']	. adm_back_link($this->u_action));
		}

		$make_words_upper_yes			= ( $new['make_words_upper'] ) ? 'checked="checked"' : '';
		$make_words_upper_no			= (!$new['make_words_upper'] ) ? 'checked="checked"' : '';
		$convert_special_letters_yes	= ( $new['convert_special_l'] ) ? 'checked="checked"' : '';
		$convert_special_letters_no		= (!$new['convert_special_l'] ) ? 'checked="checked"' : '';
		$show_stats_yes					= ( $new['show_stats'] ) ? 'checked="checked"' : '';
		$show_stats_no					= (!$new['show_stats'] ) ? 'checked="checked"' : '';
		$rules_enable_yes				= ( $new['hangman_rules_enable'] ) ? 'checked="checked"' : '';
		$rules_enable_no				= (!$new['hangman_rules_enable'] ) ? 'checked="checked"' : '';
		$requessing_lose_yes			= ( $new['requessing_lose'] ) ? 'checked="checked"' : '';
		$requessing_lose_no				= (!$new['requessing_lose'] ) ? 'checked="checked"' : '';
		$hsc_letters_p_yes				= ( $new['hsc_letters_p'] ) ? 'checked="checked"' : '';
		$hsc_letters_p_no				= (!$new['hsc_letters_p'] ) ? 'checked="checked"' : '';
		$allow_bbcode_yes				= ( $new['allow_bbcode'] ) ? 'checked="checked"' : '';
		$allow_bbcode_no				= (!$new['allow_bbcode'] ) ? 'checked="checked"' : '';
		$allow_smilies_yes				= ( $new['allow_smilies'] ) ? 'checked="checked"' : '';
		$allow_smilies_no				= (!$new['allow_smilies'] ) ? 'checked="checked"' : '';
		$show_numbers_yes				= ( $new['show_numbers'] ) ? 'checked="checked"' : '';
		$show_numbers_no				= (!$new['show_numbers'] ) ? 'checked="checked"' : '';
		$creator_bonus_yes				= ( $new['creator_bonus'] ) ? 'checked="checked"' : '';
		$creator_bonus_no				= (!$new['creator_bonus'] ) ? 'checked="checked"' : '';
		$anonymous_points_yes			= ( $new['anonymous_points'] ) ? 'checked="checked"' : '';
		$anonymous_points_no			= (!$new['anonymous_points'] ) ? 'checked="checked"' : '';
		$hangman_bot_yes				= ( $new['hangman_bot'] ) ? 'checked="checked"' : '';
		$hangman_bot_no					= (!$new['hangman_bot'] ) ? 'checked="checked"' : '';
		$import_bonus_yes				= ( $new['import_bonus'] ) ? 'checked="checked"' : '';
		$import_bonus_no				= (!$new['import_bonus'] ) ? 'checked="checked"' : '';

		//
		// FILTER STUFF FOR FILTER IN OVERVIEW PAGE
		//
		$filters = array(
			'none'		=> '<option value="none">' . $this->user->lang['FILTER_NOFILTER'].'</option>',
			'unquessed'	=> '<option value="unquessed">' . $this->user->lang['FILTER_UNQUESSED'].'</option>',
			'time_up'	=> '<option value="time_up">' . $this->user->lang['FILTER_TIME_UP'].'</option>',
			'won'		=> '<option value="won">' . $this->user->lang['FILTER_WON'].'</option>',
			'own'		=> '<option value="own">' . $this->user->lang['FILTER_OWN'].'</option>',
			'others'	=> '<option value="others">' . $this->user->lang['FILTER_OTHERS'].'</option>',
		);

		$filter = $new['standart_filter'];

		switch ($filter)
		{
			case 'unquessed':
				$filters['unquessed'] = '<option value="unquessed" selected="selected">' . $this->user->lang['FILTER_UNQUESSED'].'</option>';
			break;
			case 'time_up':
				$filters["time_up"] = '<option value="time_up" selected="selected">' . $this->user->lang['FILTER_TIME_UP'].'</option>';
			break;
			case 'won':
				$filters["won"] = '<option value="won" selected="selected">' . $this->user->lang['FILTER_WON'].'</option>';
			break;
			case 'own':
				$filters["own"] = '<option value="own" selected="selected">' . $this->user->lang['FILTER_OWN'].'</option>';
			break;
			case 'others':
				$filters["others"] = '<option value="others" selected="selected">' . $this->user->lang['FILTER_OTHERS'].'</option>';
			break;
			default:
				$filters["none"] = '<option value="none" selected="selected">' . $this->user->lang['FILTER_NOFILTER'].'</option>';
			break;
		}

		$filters_list = '';

		foreach ($filters as $text)
		{
			$filters_list .= $text;
		}

		//Higscore filter
		$higscore_filters = array(
			'won'			=>	$this->user->lang['ORDER_BY_WON'],
			'lost' 			=>	$this->user->lang['ORDER_BY_LOST'],
			'score' 		=>	$this->user->lang['ORDER_BY_POINTS'],
			'created' 		=>	$this->user->lang['ORDER_BY_CREATED'],
			'r_letters' 	=>	$this->user->lang['ORDER_BY_R_LETTERS'],
			'w_letters'		=>	$this->user->lang['ORDER_BY_W_LETTERS'],
		);
		$highscore_filter_list = '';

		foreach ($higscore_filters as $key => $var)
		{
			$highscore_filter_list .= '<option value="' . $key . '" ' . ($key == $hangman_config['highscore_filter'] ? 'selected="selected"' : '') . '>' . $var . '</option>';
		}

		// REST OF TEMPLATE -.-
		$this->template->assign_vars(array(
			'S_CONFIG'							=> true,
			'L_HANGMAN_ENABLE_YES'				=> $this->config['hangman_enable'] ? 'checked="checked"' : '',
			'L_HANGMAN_ENABLE_NO'				=> !$this->config['hangman_enable'] ? 'checked="checked"' : '',
			'FILTERS_LIST'						=> $filters_list,
			'HIGHSCORE_FILTER_LIST'				=> $highscore_filter_list,
			'L_GAMES_PER_PAGE'					=> $new['games_per_page'],
			'L_TIME_PLAY_AGAIN'					=> $new['time_play_again'],
			'L_STANDART_TITLE'					=> $new['standart_title'],
			'L_MIN_HELP'						=> $new['min_help'],
			'L_STANDART_DAYS'					=> $new['standart_days'],
			'L_STANDART_TRIES'					=> $new['standart_tries'],
			'L_MINIMUM_TRIES'					=> $new['minimum_tries'],
			'L_MAXIMUM_TRIES'					=> $new['maximum_tries'],
			'L_MINIMUM_LETTERS'					=> $new['minimum_letters'],
			'L_MAXIMUM_LETTERS'					=> $new['maximum_letters'],
			'L_ALLOW_BBCODE_YES' 				=> $allow_bbcode_yes,
			'L_ALLOW_BBCODE_NO'					=> $allow_bbcode_no,
			'L_ALLOW_SMILIES_YES' 				=> $allow_smilies_yes,
			'L_ALLOW_SMILIES_NO'				=> $allow_smilies_no,
			'L_GAMES_PER_DAY'					=> $new['games_per_day'],
			'L_GAMES_PER_DAY'					=> $new['games_per_day'],
			'L_SHOW_STATS_YES' 					=> $show_stats_yes,
			'L_SHOW_STATS_NO'					=> $show_stats_no,
			'L_SHOW_NUMBERS_YES' 				=> $show_numbers_yes,
			'L_SHOW_NUMBERS_NO'					=> $show_numbers_no,
			'L_RULES_ENABLE_YES' 				=> $rules_enable_yes,
			'L_RULES_ENABLE_NO'					=> $rules_enable_no,
			'L_MAKE_WORDS_UPPER_YES' 			=> $make_words_upper_yes,
			'L_MAKE_WORDS_UPPER_NO'				=> $make_words_upper_no,
			'L_CONVERT_SPECIAL_LETTERS_YES' 	=> $convert_special_letters_yes,
			'L_CONVERT_SPECIAL_LETTERS_NO'		=> $convert_special_letters_no,
			'L_SCORE_WON'						=> $new['score_won'],
			'L_SCORE_LOST'						=> $new['score_lost'],
			'L_SCORE_CREATE'					=> $new['score_create'],
			'L_SCORE_LETTERS'					=> $new['score_letters'],
			'L_HSC_LETTERS_P_YES' 				=> $hsc_letters_p_yes,
			'L_HSC_LETTERS_P_NO'				=> $hsc_letters_p_no,
			'SEL_' . $new['highscore_filter']	=> 'selected="selected"',
			'L_ALLOWED_LETTERS'					=> $new['letters'],
			'L_AUTO_REPLACE_LETTERS'			=> $new['auto_replace_letters'],
			'L_REQUESSING_LOSE_YES'				=> $requessing_lose_yes,
			'L_REQUESSING_LOSE_NO'				=> $requessing_lose_no,
			'L_HANGMAN_CREATOR_BONUS_YES'		=> $creator_bonus_yes,
			'L_HANGMAN_CREATOR_BONUS_NO'		=> $creator_bonus_no,
			'L_CREATOR_BONUS_SIZE'				=> $new['creator_bonus_size'],
			'L_HANGMAN_ANONYMOUS_POINTS_YES'	=> $anonymous_points_yes,
			'L_HANGMAN_ANONYMOUS_POINTS_NO'		=> $anonymous_points_no,
			'L_HANGMAN_BOT_NAME'				=> $new['bot_name'],
			'L_HANGMAN_BOT_YES'				 	=> $hangman_bot_yes,
			'L_HANGMAN_BOT_NO'				 	=> $hangman_bot_no,
			'L_IMPORT_BONUS_YES'				=> $import_bonus_yes,
			'L_IMPORT_BONUS_NO'					=> $import_bonus_no,
			'L_IMPORT_BONUS_SIZE'				=> $new['import_bonus_size'],
			'U_ACTION'							=> $this->u_action,
		));
	}
	/**
	* Set page url
	*
	* @param string $u_action Custom form action
	* @return null
	* @access public
	*/
	public function set_page_url($u_action)
	{
		$this->u_action = $u_action;
	}
}
	