<?php
/**
*
* @package phpBB Extension - Hangman Game
* @copyright (c) 2017 dmzx - http://www.dmzx-web.net
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace dmzx\hangmangame\controller;

define('HANGMAN_GERMAN_LETTERS', 'ÄÖÜß');

class main
{
	/** @var \dmzx\hangmangame\core\functions_hangmangame */
	protected $functions;

	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\user */
	protected $user;

	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\controller\helper */
	protected $helper;

	/** @var \phpbb\pagination */
	protected $pagination;

	/** @var string phpBB root path */
	protected $root_path;

	/** @var string php_ext */
	protected $php_ext;

	/**
	* The database tables
	*
	* @var string
	*/
	protected $hangmangame_table_hangman_quess;

	protected $hangmangame_table_hangman_config;

	protected $hangmangame_table_hangman_words;

	protected $hangmangame_table_hangman_score;

	/**
	* Constructor
	*
	* @param \dmzx\hangmangame\core\functions_hangmangame	$functions
	* @param \phpbb\template\template		 				$template
	* @param \phpbb\user									$user
	* @param \phpbb\auth\auth								$auth
	* @param \phpbb\db\driver\driver_interface				$db
	* @param \phpbb\request\request		 					$request
	* @param \phpbb\config\config							$config
	* @param \phpbb\controller\helper		 				$helper
	* @param \phpbb\pagination								$pagination
	* @param string 										$root_path
	* @param string 										$php_ext
	* @param string 										$hangmangame_table_hangman_quess
	* @param string 										$hangmangame_table_hangman_config
	* @param string 										$hangmangame_table_hangman_words
	* @param string 										$hangmangame_table_hangman_score
	*
	*/
	public function __construct(
		\dmzx\hangmangame\core\functions_hangmangame $functions,
		\phpbb\template\template $template,
		\phpbb\user $user,
		\phpbb\auth\auth $auth,
		\phpbb\db\driver\driver_interface $db,
		\phpbb\request\request $request,
		\phpbb\config\config $config,
		\phpbb\controller\helper $helper,
		\phpbb\pagination $pagination,
		$root_path,
		$php_ext,
		$hangmangame_table_hangman_quess,
		$hangmangame_table_hangman_config,
		$hangmangame_table_hangman_words,
		$hangmangame_table_hangman_score
	)
	{
		$this->functions 						= $functions;
		$this->template 						= $template;
		$this->user 							= $user;
		$this->auth 							= $auth;
		$this->db 								= $db;
		$this->request 							= $request;
		$this->config 							= $config;
		$this->helper 							= $helper;
		$this->pagination 						= $pagination;
		$this->root_path 						= $root_path;
		$this->php_ext 							= $php_ext;
		$this->hangmangame_table_hangman_quess	= $hangmangame_table_hangman_quess;
		$this->hangmangame_table_hangman_config = $hangmangame_table_hangman_config;
		$this->hangmangame_table_hangman_words 	= $hangmangame_table_hangman_words;
		$this->hangmangame_table_hangman_score 	= $hangmangame_table_hangman_score;
	}

	public function handle_hangmangame()
	{
		// Is Board Hangman enable?
		if (!$this->config['hangman_enable'])
		{
			meta_refresh(3, append_sid($this->root_path . 'index.' . $this->php_ext));
			trigger_error($this->user->lang['HANGMAN_BOARD_DISABLE']);
		}

		$hangman_config = $this->functions->hangman_config();

		$this->functions->assign_authors();
		$this->template->assign_var('HANGMANGAME_FOOTER_VIEW', true);

		$this->template->assign_vars(array(
			'HANGMANGAME_FOOTER_VIEW'	=> true,
			'HANGMANGAME_VERSION'		=> $this->config['hangmangame_version']
		));

		// Make sure the player is registered if not AND $hangman_config says he can't see any...
		$user_id = $this->user->data['user_id'];

		// Sorry, only logged users...
		if (!$this->user->data['is_registered'] && !$this->auth->acl_get('u_hangman_play'))
		{
			if ($this->user->data['is_bot'])
			{
				redirect(append_sid("{$this->root_path}index.$this->php_ext"));
			}

			login_box('', $this->user->lang['LOGIN_INFO']);
		}

		$page_title = $this->user->lang['HANGMAN'];

		// Get submitted params ;)
		$mode 				= $this->request->variable('mode', '');
		$order				= $this->request->variable('order', '');
		$buchstabe			= $this->request->variable('buchstabe', '', true);
		$start				= $this->request->variable('start', 0);
		$filter				= $this->request->variable('filter', $hangman_config['standart_filter']);
		$hlimit				= $this->request->variable('hlimit', 10);
		$game_index			= $this->request->variable('game_index', 0);
		$maximal_versuche	= $this->request->variable('maximal_versuche', 0);
		$wort				= $this->request->variable('wort', '', true);
		$wortversuch		= $this->request->variable('wortversuch', '', true);
		$subject			= $this->request->variable('subject', '', true);
		$hilfe				= $this->request->variable('hilfe', '', true);
		$gueltige_tage		= $this->request->variable('gueltige_tage', 0);
		$confirm			= $this->request->variable('confirm', '', true);
		$hfilter			= $this->request->variable('hfilter', '');
		$horder				= $this->request->variable('horder', '');

		$board_url = generate_board_url() . '/';

		add_form_key('hangman_key');

		//
		// PARANOIA PARANOIA PARANOIA PARANOIA PARANOIA PARANOIA PARANOIA PARANOIA PARANOIA
		// Some security checks on vars
		//
		if ($horder !='asc' && $horder!='desc')
		{
			$horder = 'desc';
		}

		if ($hfilter != 'won' && $hfilter !='lost' && $hfilter!='score' && $hfilter!='created' && $hfilter!='r_letters' && $hfilter!='w_letters')
		{
			$hfilter = '';
		}

		if ($buchstabe != '')
		{
			if (utf8_strlen($buchstabe) > 3)
			{
				exit;
			}

			// If config says -> convert to upper...
			if ($hangman_config['make_words_upper'])
			{
				$buchstabe = utf8_strtoupper($buchstabe);
			}
		}

		if ($filter)
		{
			if ($filter!='' && $filter!='unquessed' && $filter!='won' && $filter!='own' && $filter!='others' && $filter!='time_up' && $filter!='none')
			{
				exit;
			}
		}
		//
		// END PARANOIA END PARANOIA END PARANOIA END PARANOIA END PARANOIA END PARANOIA END PARANOIA
		//

		//Template variables for the navigation
		$this->template->assign_block_vars('navlinks', array(
			'FORUM_NAME'	=> $this->user->lang['HANGMAN'],
			'U_VIEW_FORUM'	=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')),
		));

		// Standard site == overview (if no mode submitted - hangman.php with no post/get-vars)
		if (!$mode)
		{
			$mode = 'overview';
		}

		if ($mode == 'overview')
		{
			$this->template->set_filenames(array(
				'body' => 'hangman/hangman_overview.html',
			));

			// Is there any filter set?
			// MAKE A FILTER
			$filter_sql = '';
			$page_nav = '';

			$filters = array(
				'none'		=> '<option value="none">' . $this->user->lang['FILTER_NOFILTER'] . '</option>',
				'unquessed'	=> '<option value="unquessed">' . $this->user->lang['FILTER_UNQUESSED'] . '</option>',
				'time_up'	=> '<option value="time_up">' . $this->user->lang['FILTER_TIME_UP'] . '</option>',
				'won'		=> '<option value="won">' . $this->user->lang['FILTER_WON'] . '</option>',
				'own'		=> '<option value="own">' . $this->user->lang['FILTER_OWN'] . '</option>',
				'others'	=> '<option value="others">' . $this->user->lang['FILTER_OTHERS'] . '</option>'
			);

			switch ($filter)
			{
				case 'unquessed':
					$page_nav = "&amp;submit=submit&amp;filter=unquessed";
					$now = time();
					$filter_sql = " WHERE bwon = '0' AND NOT (time_limit <> '0' AND time_limit < " . $now .")";
					$filters['unquessed'] = '<option value="unquessed" selected="selected">' . $this->user->lang['FILTER_UNQUESSED'].'</option>';
				break;
				case 'time_up':
					$page_nav = "&amp;submit=submit&amp;filter=time_up";
					$now = time();
					$filter_sql = " WHERE time_limit < '$now' AND bwon <> '1' AND time_limit <> '0'";
					$filters["time_up"] = '<option value="time_up" selected="selected">' . $this->user->lang['FILTER_TIME_UP'].'</option>';
				break;
				case 'won':
					$page_nav = "&amp;submit=submit&amp;filter=won";
					$filter_sql = " WHERE bwon = '1'";
					$filters["won"] = '<option value="won" selected="selected">' . $this->user->lang['FILTER_WON'] . '</option>';
				break;
				case 'own':
					$page_nav = "&amp;submit=submit&amp;filter=own";
					$filter_sql = " WHERE user_id = ". (int) $user_id;
					$filters["own"] = '<option value="own" selected="selected">' . $this->user->lang['FILTER_OWN'] . '</option>';
				break;
				case 'others':
					$page_nav = "&amp;submit=submit&amp;filter=others";
					$filter_sql = " WHERE user_id <> " . (int) $user_id;
					$filters["others"] = '<option value="others" selected="selected">' . $this->user->lang['FILTER_OTHERS'] . '</option>';
				break;
				default:
					$page_nav = '';
					$filter_sql = '';
					$filters["none"] = '<option value="none" selected="selected">' . $this->user->lang['FILTER_NOFILTER'] . '</option>';
				break;
			}

			$per_page = $hangman_config['games_per_page'];

			// ADMIN or MODERATOR?
			if ($this->auth->acl_get('a_hangman_set') || $this->auth->acl_get('m_hangman_delete'))
			{
				$this->template->assign_vars(array(
					'U_ADMIN_MULTIDELETE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'multi_del')),
					'S_ADMIN_MULTIDELETE_GO'	=> $this->user->lang['ACP_MULTIDELETE_GO'],
					'S_ADMIN_MULTIDELETE_RESET'	=> $this->user->lang['ACP_RESET']
				));
			}

			/** Show numbers # in front of hangman
			Looking for HANGMAN_DATE, HANGMAN_TOPIC, HANGMAN_CREATOR, HANGMAN_STATUS
			for creating main page overview filtered by filter limited by hangman_cfg
			*/
			$sql = 'SELECT *
				FROM ' . $this->hangmangame_table_hangman_words . " $filter_sql
				ORDER BY bwon ASC, time_created DESC, bwon ASC ";
			$result = $this->db->sql_query_limit($sql, $per_page, $start);

			$i = 0;
			// Assign vars for each hangman
			while ($row = $this->db->sql_fetchrow($result))
			{
				// General data
				$hangman_date		= $row['time_created'];
				$hangman_lease_date	= $row['time_limit'];
				$hangman_max_quess	= $row['max_tries'];
				$hangman_topic		= $row['hangman_subject'];
				$hangman_id			= $row['hangman_id'];
				$hangman_word		= $row['hangman_word'];

				// If hangman subject is to long and not bbcoded cut it to 20 chars
				if (utf8_strlen(strip_tags($hangman_topic)) > 25 && !stristr($hangman_topic, '[/img') && !stristr($hangman_topic, '[/color') && !stristr($hangman_topic, '[/b') && !stristr($hangman_topic, '[/i') && !stristr($hangman_topic, '[/size'))
				{
					$hangman_topic = utf8_substr($hangman_topic, 0, 25) .'...';
				}

				// Get username of creator
				$hangman_creator = (($row['user_id'] == ANONYMOUS || $row['user_id'] == 0) && $hangman_config['hangman_bot']) ? $hangman_config['bot_name'] : $this->functions->get_username($row['user_id']);
				$users_losttime = $this->functions->get_lost_time($user_id, $hangman_id);
				// Display hangman status in overview
				$hangman_status = $this->user->lang['STATUS_GREEN'];

				if (((time() - $users_losttime) < 60 * $hangman_config['time_play_again']))
				{
					$zeit = 60 * $hangman_config['time_play_again'] - (time() - $users_losttime);

					if ($zeit > 7200)
					{
						$zeit = ($zeit - ($zeit%3600)) / 3600;
						$quess_msg = sprintf($this->user->lang['QUESS_LOST_RETRY_OVERVIEW'], $zeit, $this->user->lang['HOURS']);
					}
					else if ($zeit > 60)
					{
						$zeit = ($zeit - ($zeit%60)) / 60;
						$quess_msg = sprintf($this->user->lang['QUESS_LOST_RETRY_OVERVIEW'], $zeit, $this->user->lang['MINUTES']);
					}
					else
					{
						$quess_msg = sprintf($this->user->lang['QUESS_LOST_RETRY_OVERVIEW'], $zeit, $this->user->lang['SECONDS']);
					}
					$hangman_status = $quess_msg;
				}
				else if ($hangman_config['time_play_again'] == 0 && $users_losttime)
				{
					if ($mode == 'window')
					{
						$hangman_status = $this->user->lang['QUESS_LOST_WIN'];
					}
					else
					{
						$hangman_status = sprintf($this->user->lang['QUESS_LOST'], $backlink, '</a>');
					}
				}

				if ($hangman_lease_date > 0)
				{
					$days_left = @gmdate("d", $hangman_lease_date - time());
				}

				// If hangman is out of time nobody can quess it now ;)
				if ($hangman_lease_date != 0 && ($hangman_lease_date < time()))
				{
					$hangman_status = sprintf($this->user->lang['STATUS_TIME'], $hangman_word);
					$days_left = gmdate('d.m.y', $hangman_lease_date);
				}
				// If hangman is not limited assign unlimited status to days left
				else if ($hangman_lease_date == 0)
				{
					$days_left = $this->user->lang['STATUS_UNLIMITED'];
				}
				// If not display remaining time
				else
				{
					$days_left = gmdate("d", $hangman_lease_date-time());
				}

				// Is the game won?
				if ($row['bwon'])
				{
					// If the hangman is won > the time is changed to lonely '---'
					$days_left = '---';
					// Who is the winner?
					$errater = $this->functions->get_username($row['winner']);
					$hangman_status = sprintf($this->user->lang['STATUS_WON'], (utf8_strlen($hangman_word) < 15 ? $hangman_word : utf8_substr($hangman_word,0 , 12) . '...'), $errater);
				}

				// For different colors in different lines
				$klasse = (($i%2 == 1) ? "row1" : "row3");

				// Get hangman quessors
				$quessors = array();
				$quessor_count = 0;
				$quessors_count = $this->functions->get_quessors_count($row['user_id'], $hangman_id);
				$quessors = $this->functions->get_quessors($row['user_id'], $hangman_id);
				$lang_quessors = ($quessors_count > 0) ? $this->user->lang['QUESS_QUESSORS'] : $this->user->lang['QUESS_NOBODY'];

				$this->template->assign_block_vars('hangmans', array(
					'KLASSE'				=> $klasse,
					'HANGMAN_ID'			=> $hangman_id,
					'HANGMAN_DATE'			=> $this->user->format_date($hangman_date, false, false),
					'HANGMAN_LEASE_DATE'	=> $days_left,
					'HANGMAN_TOPIC'			=> censor_text(generate_text_for_display($hangman_topic, $row['bbcode_uid'], $row['bbcode_bitfield'], $row['bbcode_flags'])),
					'HANGMAN_LINK'			=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'quess', 'game_index' => $hangman_id, 'start' =>	$start)),
					'HANGMAN_CREATOR'		=> $hangman_creator,
					'HANGMAN_MAX_QUESS'		=> $hangman_max_quess,
					'HANGMAN_STATUS'		=> $hangman_status,
					'HANGMAN_ADMIN_DELETE'	=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'delete', 'game_index' => $hangman_id)),
					'L_QUESSORS_TEXT'		=> $lang_quessors,
					'U_WINDOW'				=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'window', 'game_index' => $hangman_id, 'start' => $start)),
				));

				// Assign quessors
				for ($j = 0; $j < $quessors_count; $j++)
				{
					$quessors_name	= $quessors[$j]['name'] . ((intval($quessors[$j]['quess_count']) > 1) ? '(' . intval($quessors[$j]['quess_count']) . ')':'');

					if ($quessors[$j]['user_id'] == $user_id)
					{
						$quessors_name = '<strong>' . $quessors_name . '</strong>';
					}

					if ($quessors_count - 1 != $j)
					{
						$quessors_name	= $quessors_name . ',';
					}
					else
					{
						$quessors_name	= $quessors_name;
					}

					$this->template->assign_block_vars('hangmans.quessors', array(
						'L_QUESSORS_NAME'	=>	$quessors_name,
					));
				}
				$i++;
			} // End assigning for each hangman

			$this->db->sql_freeresult($result);

			// Filter
			$filter_list = '';

			foreach ($filters as $text)
			{
				$filter_list .= $text;
			}

			// Ministatistics
			if ($hangman_config['show_stats'])
			{
				$stats = $this->functions->get_stats();
				$this->template->assign_vars(array(
					'L_HANGMANSTATS_ONLINE'	=> sprintf($this->user->lang['HANGMANSTATS_ONLINE'], $stats['count']),
					'L_HANGMANSTATS_TODAY'	=> sprintf($this->user->lang['HANGMANSTATS_TODAY'], $stats['today']),
					'L_HANGMANSTATS_WON'	=> sprintf($this->user->lang['HANGMANSTATS_WON'], $stats['won']),
					'L_HANGMANSTATS_BEST'	=> sprintf($this->user->lang['HANGMANSTATS_BEST'], $this->functions->get_username($stats['best_player'])),
					'L_HANGMANSTATS_TOTAL'	=> sprintf($this->user->lang['HANGMANSTATS_TOTAL'], $stats['total_hangmans'])
				));
			}

			$sql = 'SELECT COUNT(hangman_id) AS total
				FROM ' . $this->hangmangame_table_hangman_words . '	' . $filter_sql;
			$result = $this->db->sql_query($sql);
			$row = $this->db->sql_fetchrow($result);
			$total_hangmens = $row['total'];
			$this->db->sql_freeresult($result);

			$pagination_url = $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview', 'filter' => $filter));
			//Start pagination
			$this->pagination->generate_template_pagination($pagination_url, 'pagination', 'start', $total_hangmens, $per_page, $start);

			// Template assign the remaining vars
			$this->template->assign_vars(array(
				'S_CAN_PLAY_HANGMAN'	=> $this->auth->acl_get('u_hangman_play') ? true : false,
				'S_HANGMAN_ADMIN'		=> $this->auth->acl_get('a_hangman_set') || $this->auth->acl_get('m_hangman_delete') ? true : false,
				'S_SHOW_STATS'			=> $hangman_config['show_stats'] ? true : false,
				'S_SHOW_NUMBERS'		=> $hangman_config['show_numbers'] ? true : false,
				'FILTER_LIST'			=> $filter_list,
				'U_HIGHSCORE_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore')),
				'U_CREATION_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'create')),
				'U_HELP_PAGE'			=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'help')),
				'U_OVERVIEW_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')),
				'L_FILTER_ACTION'		=> $this->helper->route('dmzx_hangmangame_controller'),
				'DELETE_IMG' 			=> $this->user->img('icon_post_delete', 'ACP_DELETE'),
				'TOTAL_HANGMANS'		=> $this->user->lang('TOTAL_HANGMANS', (int) $total_hangmens),
			));
			page_header($page_title);
			page_footer();
		}
		else if ($mode == 'quess' || $mode == 'window')
		{
			$quess_picture = '';

			if ($mode == 'window')
			{
				$page_title = $this->user->lang['HANGMAN'];
				$gen_simple_header = true;

				$this->template->set_filenames(array(
					'body' => 'hangman/hangman_window.html'
				));
			}
			else
			{
				$this->template->set_filenames(array(
					'body' => 'hangman/hangman_quess.html'
				));

				//Template variables for the navigation
				$this->template->assign_block_vars('navlinks', array(
					'FORUM_NAME'	=> $this->user->lang['HANGMAN_QUESS'],
					'U_VIEW_FORUM'	=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'quess', 'game_index' => $game_index, 'start' =>	$start)),
				));
			}

			//Back link to overview if game over
			$backlink = '<a href="' . $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview', 'start' =>	$start)) . '">';

			// Is there a hangman with the given index?
			$sql = 'SELECT *
				FROM ' . $this->hangmangame_table_hangman_words . '
				WHERE hangman_id = ' . (int) $game_index;
			$result = $this->db->sql_query($sql);

			if (!($row = $this->db->sql_fetchrow($result)))
			{
				trigger_error($this->user->lang['ERROR_GET_HANGMAN']);
			}

			if (!$this->auth->acl_get('u_hangman_play'))
			{
				meta_refresh(3, $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview', 'start' =>	$start)));
				trigger_error($this->user->lang['HANGMAN_NO_PLAY_PERMISSION']);
			}

			$this->db->sql_freeresult($result);

			//Set some vars
			$hangman_word 		= $row['hangman_word'];
			$hangman_bbcode 	= $row['bbcode_uid'];
			$hangman_bitfield 	= $row['bbcode_bitfield'];
			$hangman_flags 		= $row['bbcode_flags'];
			$help_bbcode 		= $row['help_bbcode_uid'];
			$help_bitfield 		= $row['help_bbcode_bitfield'];
			$help_flags 		= $row['help_bbcode_flags'];

			if ($hangman_config['make_words_upper'])
			{
				$hangman_word	= utf8_strtoupper($hangman_word);
			}

			$hangman_subject			= generate_text_for_display($row['hangman_subject'], $row['bbcode_uid'], $row['bbcode_bitfield'], $row['bbcode_flags']);
			$hangman_msg				= $row['hangman_help'];
			$hangman_quessed_letters	= $row['quessed_letters'];
			$hangman_msg				= str_replace("\n", "\n<br />\n", $hangman_msg);
			$hangman_max_versuche		= $row['max_tries'];
			$hangman_date				= $row['time_created'];
			$hangman_lease_date			= $row['time_limit'];
			$hangman_creator_id			= $row['user_id'];

			// Get the users who tried to quess
			$quessors 		= array();
			$quessor_count	= 0;
			$quessors_count = $this->functions->get_quessors_count($row['user_id'], $game_index);
			$quessors = $this->functions->get_quessors($row['user_id'], $game_index);
			$lang_quessors = ($quessors_count > 0) ? $this->user->lang['QUESS_QUESSORS'] : $this->user->lang['QUESS_NOBODY'];

			$this->template->assign_vars(array(
				'L_QUESSORS_TEXT'	=> $this->user->lang['QUESS_QUESSORS'],
			));

			// Assign quessors
			for ($i = 0; $i < $quessors_count; $i++)
			{
				$quessors_name	= $quessors[$i]['name'] . ((intval($quessors[$i]['quess_count']) > 1) ? '(' . intval($quessors[$i]['quess_count']) . ')':'');
				if ($quessors[$i]['user_id'] == $user_id)
				{
					$quessors_name = '<strong>' . $quessors_name . '</strong>';
				}

				if ($quessors_count - 1 != $i)
				{
					$quessors_name	= $quessors_name . ', ';
				}
				else
				{
					$quessors_name	= $quessors_name;
				}

				$this->template->assign_block_vars('quessors', array(
					'L_QUESSORS_NAME'	=>	$quessors_name,
				));
			}

			// Get creators name
			$bdone_play = 'no';
			$hangman_creator = (($hangman_creator_id == ANONYMOUS || !$hangman_creator_id ) && $hangman_config['hangman_bot']) ? $hangman_config['bot_name'] : $this->functions->get_username($hangman_creator_id);

			// Word already quessed
			if ($row['bwon'] && $bdone_play == 'no')
			{
				// Who is the winner?
				$username = $this->functions->get_username($row['winner']);
				// Make word ready
				$hangman_word = str_replace('_', '&#8226;', utf8_wordwrap($hangman_word, 1, ' ',1));

				if ($mode == 'window')
				{
					$quess_msg = sprintf($this->user->lang['QUESS_WON_OTHER_WIN'], $username);
				}
				else
				{
					$quess_msg = sprintf($this->user->lang['QUESS_WON_OTHER'], $username, $backlink, '</a>');
				}
				$bdone_play = 'disp_word';
			}

			// Creator is trying to quess ?
			if ($user_id == $row['user_id'] && $bdone_play == 'no')
			{
				// Add the quessed letters to show author how good the others are, but only if not won yet
				if (!$row['bwon'])
				{
					$this->functions->quessed_letters($hangman_word, $hangman_quessed_letters);
				}

				$hangman_word = str_replace('_','&#8226;', utf8_wordwrap($hangman_word , 1, ' ', 1));
				$bdone_play = 'disp_word';
				$quess_msg = $this->user->lang['HANGMAN_SELF'];
			}

			// Time is up ?
			if ($hangman_lease_date != 0 && ($hangman_lease_date < time()) && $bdone_play == 'no')
			{
				// Space displaying as a dot
				$hangman_word = str_replace('_','&#8226;', utf8_wordwrap($hangman_word, 1,' ', 1));
				$quess_msg = $this->user->lang['QUESS_TIME_UP'];
				$quess_picture = '<img src="' . $board_url . 'ext/dmzx/hangmangame/images/hangman_pics/time.gif" alt="" />';
				$bdone_play = 'disp_word';
			}

			if ($bdone_play == 'no')
			{
				// Already started or need a new entry @ hangman_quess ?...
				$sql = 'SELECT *
					FROM ' . $this->hangmangame_table_hangman_quess . '
					WHERE hangman_id = ' . (int) $game_index . '
						AND user_id = ' . (int) $user_id;
				$result = $this->db->sql_query($sql);
				$row = $this->db->sql_fetchrow($result);
				$this->db->sql_freeresult($result);
				$hangman_versuche = $row['tries'];

				if ($row['user_id'] != $user_id)
				{
					//... we need one...
					$sql_ary = array(
						'user_id'				=> (int) $user_id,
						'hangman_id'			=> (int) $game_index,
						'tries'					=> 0,
						'lost_time'				=> 0,
						'quess_count'			=> 1,
					);

					$sql = 'INSERT INTO ' . $this->hangmangame_table_hangman_quess . ' ' . $this->db->sql_build_array('INSERT', $sql_ary);
					$this->db->sql_query($sql);

					$sql = 'SELECT *
						FROM ' . $this->hangmangame_table_hangman_quess . '
						WHERE hangman_id = ' . (int) $game_index . '
							AND user_id = ' . (int) $user_id;
					$result = $this->db->sql_query($sql);
					$row = $this->db->sql_fetchrow($result);
					$this->db->sql_freeresult($result);

					$hangman_versuche = 0;
					// To prevent add 1 to tries on getting first time on hangman ;)
					$hangman_config['requessing_lose'] = false;
				}

				// Too much?
				if ($hangman_versuche >= $hangman_max_versuche)
				{
					$bbreack = false;

					// Yes, to much, but could he do it again?
					if ($hangman_config['time_play_again'] != 0)
					{
						$hangman_losttime = $row['lost_time'];

						// Time difference ? Can user play again?
						if ((time() - $hangman_losttime) > 60 * $hangman_config['time_play_again'])
						{
							// Huuu he waited so long ...	let him play again
							$quesscount = $row['quess_count'] + 1;
							$sql = 'UPDATE ' . $this->hangmangame_table_hangman_quess . "
								SET tries = 0, lost_time = 0, quess_count = $quesscount
								WHERE hangman_id = $game_index
									AND user_id = " . $user_id;
							$result = $this->db->sql_query($sql);
							$bbreack = true;
							$hangman_versuche = 0;
							// To prevent add 1 to tries on getting first time on the hangman ;)
							$hangman_config['requessing_lose'] = false;
						}
					}

					// User will play again or not ?
					if ($bbreack == false)
					{
						// Display all quessed stuff
						$this->functions->quessed_letters($hangman_word, $hangman_quessed_letters);
						// Message and if he can play again...
						$quess_msg = sprintf($this->user->lang['QUESS_LOST'], $backlink, '</a>');

						if ($hangman_config['time_play_again'] != 0)
						{
							$zeit = 60 * $hangman_config['time_play_again'] - (time() - $hangman_losttime);

							if ($zeit > 7200)
							{
								$zeit = ($zeit - ($zeit%3600)) / 3600;
								$quess_msg = sprintf($this->user->lang['QUESS_LOST_RETRY'], $zeit, $this->user->lang['HOURS']);
							}
							else if ($zeit > 60)
							{
								$zeit = ($zeit - ($zeit%60)) / 60;
								$quess_msg = sprintf($this->user->lang['QUESS_LOST_RETRY'], $zeit, $this->user->lang['MINUTES']);
							}
							else
							{
								$quess_msg = sprintf($this->user->lang['QUESS_LOST_RETRY'], $zeit, $this->user->lang['SECONDS']);
							}

							$quess_picture = '<img src="' . $board_url . 'ext/dmzx/hangmangame/images/hangman_pics/trans_lost_' . 9 . '.gif" alt="" />';
							$bdone_play = 'yes';
						}
					}
				}
			}
			// Will play or time stuff end
			if ($bdone_play == 'no')
			{
				//... letter or word submitted - check it
				$bfound = false;
				$counter = -1;

				// Word?
				if (utf8_strlen($wortversuch) < 0 || (utf8_strlen($wortversuch) > $hangman_config['maximum_letters'] && $hangman_config['maximum_letters'] != -1))
				{
					$wortversuch = '';
				}

				if ($wortversuch != '')
				{
					if (!check_form_key('hangman_key'))
					{
						trigger_error('FORM_INVALID');
					}

					if ($hangman_config['make_words_upper'] == true)
					{
						$wortversuch =	utf8_strtoupper($wortversuch);
					}
					$wortversuch = str_replace(chr(32), '_', $wortversuch);

					if ($hangman_config['make_words_upper'] == true)
					{
						if (utf8_strtoupper($hangman_word) == $wortversuch)
						{
							$bfound = true;
						}
					}
					else if ($hangman_word == $wortversuch)
					{
						$bfound = true;
					}

					if ($bfound)
					{
						$counter = $this->functions->quessed_letters($hangman_word, $hangman_word);
						$this->functions->update_score(0, 0, 0, $hangman_creator_id, $user_id, utf8_strlen($hangman_word) - utf8_strlen($hangman_quessed_letters), 0);
					}
					else
					{
						$this->functions->update_score(0, 0, 0, $hangman_creator_id, $user_id, 0, 1);
					}
				}
				// Letter if no word or wortversuch false
				if (!$bfound && $wortversuch == '')
				{
					$hangman_word_ary = utf8_str_split($hangman_word);
					for ($i = 0, $size = sizeof($hangman_word_ary); $i < $size; $i++)
					{
						$akt_char = ($hangman_config['make_words_upper']) ? utf8_strtoupper($hangman_word_ary[$i]) : $hangman_word_ary[$i];

						if ($akt_char == $buchstabe && $akt_char != '')
						{
							// Letter found
							// Just update new letters
							$bfound = true;
							if (utf8_strpos($hangman_quessed_letters, $akt_char) === false)
							{
								$hangman_quessed_letters = $hangman_quessed_letters . $akt_char;
								// First update Hangman table
								$sql = 'UPDATE ' . $this->hangmangame_table_hangman_words . "
									SET quessed_letters = '$hangman_quessed_letters'
									WHERE hangman_id = " . (int) $game_index;
								$this->db->sql_query($sql);
								$this->functions->update_score(0, 0, 0, $hangman_creator_id, $user_id, 1, 0);
							}
							else if ($hangman_config['requessing_lose'])
							{
								$bfound = false;
							}
							break;
						}
					}
					// No letter submitted > change to true > so tries will remain...
					if (!$buchstabe && !$hangman_config['requessing_lose'])
					{
						$bfound = true;
					}
				}
				// If letter not found
				if ($bfound == false)
				{
					// Update table how many failed tries
					$hangman_versuche++;
					$zeit = time();

					$sql = 'UPDATE ' . $this->hangmangame_table_hangman_quess . "
						SET tries = $hangman_versuche, lost_time = 0
						WHERE hangman_id = $game_index
							AND user_id = " . (int) $user_id;
					$this->db->sql_query($sql);

					$this->functions->update_score(0, 0, 0, $hangman_creator_id, $user_id, 0, 1);

					// How many wrong tries
					if ($hangman_versuche >= $hangman_max_versuche)
					{
						// To much > update score
						$this->functions->update_score(1, 0, 0, $hangman_creator_id, $user_id, 0, 0);
						$sql = 'UPDATE ' . $this->hangmangame_table_hangman_quess . '
							SET lost_time = ' . $zeit . '
							WHERE hangman_id = ' . (int) $game_index . '
								AND user_id = ' . (int) $user_id;
						$result = $this->db->sql_query($sql);

						// Display all quessed stuff
						$this->functions->quessed_letters($hangman_word, $hangman_quessed_letters);

						// And show all lost stuff
						if ($mode == 'window')
						{
							$quess_msg = $this->user->lang['QUESS_LOST_WIN'];
						}
						else
						{
							$quess_msg = sprintf($this->user->lang['QUESS_LOST'], $backlink, '</a>');
						}
						$quess_picture = '<img src="' . $board_url . 'ext/dmzx/hangmangame/images/hangman_pics/trans_lost_' . 9 . '.gif" alt="" />';
						$bdone_play = 'yes';
					}
				} // Checked the letter
			}
			if ($bdone_play == 'no')
			{
				if ($counter == -1)
				{
					// Display all quessed stuff and count wrong letters
					$counter = $this->functions->quessed_letters($hangman_word, $hangman_quessed_letters);
				}

				// Hangman won cause no letter is wrong
				if ($counter == 0)
				{
					if ($mode == 'window')
					{
						$quess_msg = sprintf($this->user->lang['QUESS_WON_SELF_WIN']);
					}
					else
					{
						$quess_msg = sprintf($this->user->lang['QUESS_WON_SELF'], $backlink, '</a>');
					}
					$quess_picture = '<img src="' . $board_url . 'ext/dmzx/hangmangame/images/hangman_pics/won.gif" alt="" />';
					$bdone_play = 'disp_word';

					//db update > hangman is won (version compare to avoid MySQL4 error
					$sql = 'UPDATE ' . $this->hangmangame_table_hangman_words . '
						SET bwon = 1, winner = ' . $user_id . '
						WHERE hangman_id = ' . (int) $game_index;
					$this->db->sql_query($sql);

					//update score
					$this->functions->update_score(0, 1, 0, 0, $user_id);
				}
			}

			if ($bdone_play == 'no')
			{
				$this->template->assign_block_vars('switch_show_letters', array(
					'L_QUESS_LETTERS'	=> $this->user->lang['QUESS_LETTERS'],
					'U_ACTION'			=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'quess', 'game_index' =>	$game_index, 'filter' => $filter, 'start' => $start)),
				));

				if (intval($hangman_config['maximum_letters']) > 0 && intval($hangman_config['maximum_letters']) < 255)
				{
					$this->template->assign_block_vars('switch_show_letters.switch_shorter_words', array(
						'MAXLENGTH'	=> $hangman_config['maximum_letters']
					));
				}
				else
				{
					$this->template->assign_block_vars('switch_show_letters.switch_longer_words', array());
				}

				$restversuche = $hangman_max_versuche - $hangman_versuche;
				// Display link to letters
				$letters = utf8_str_split($hangman_config['letters']);

				for ($i = 0, $size = sizeof($letters); $i < $size; $i++)
				{
					$l = urlencode($letters[$i]);
					$link = ($mode == 'window') ? $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'window', 'game_index' =>	$game_index, 'buchstabe' => $l, 'filter' => $filter, 'start' => $start)) : $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'quess', 'game_index' =>	$game_index, 'buchstabe' => $l, 'filter' => $filter, 'start' => $start));

					if ($letters[$i])
					{
						if (utf8_strpos($hangman_quessed_letters.$hangman_config['auto_replace_letters'], $letters[$i]) === false)
						{
							$this->template->assign_block_vars('switch_show_letters.letters', array(
								'LETTER_LINK'	=> $link,
								'LETTER_SIGN'	=> $letters[$i],
							));
						}
					}
					else if ($letters[$i] == '0')
					{
						if (utf8_strpos($hangman_quessed_letters.$hangman_config['auto_replace_letters'], '0') === false)
						{
							$this->template->assign_block_vars('switch_show_letters.letters', array(
								'LETTER_LINK'	=> $link,
								'LETTER_SIGN'	=> $letters[$i],
							));
						}
					}
				}
				$quess_msg = sprintf($this->user->lang['QUESS_REMAINING_TRIES'], $restversuche);

				// Get the picture ;)
				if ($restversuche >= 9)
				{
					$quess_picture = '<img src="' . $board_url . 'ext/dmzx/hangmangame/images/hangman_pics/trans_lost_' . 0 . '.gif" alt="" />';
				}
				else
				{
					$quess_picture = '<img src="' . $board_url . 'ext/dmzx/hangmangame/images/hangman_pics/trans_lost_' . (8 - $restversuche) . '.gif" alt="" />';
				}
				$bdone_play = 'yes';
			}

			if ($bdone_play == 'no')
			{
				trigger_error('Something gone wrong :-(');
			}
			else
			{
				$this->template->assign_vars(array(
					'HANGMAN_WORD'				=> ($bdone_play == 'disp_word') ? '<br /><br />' . $hangman_word : '',
					'QUESS_MSG'					=> $quess_msg,
					'QUESS_PICTURE'				=> $quess_picture,
					'HANGMAN_SUBJECT'			=> censor_text(generate_text_for_display($hangman_subject, $hangman_bbcode, $hangman_bitfield, $hangman_flags)),
					'HANGMAN_MSG'				=> censor_text(generate_text_for_display($hangman_msg, $help_bbcode, $help_bitfield, $help_flags)),
					'L_HANGMAN_CREATOR_NAME'	=> $hangman_creator,
					'U_HIGHSCORE_PAGE'			=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore')),
					'U_CREATION_PAGE'			=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'create')),
					'U_HELP_PAGE'				=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'help')),
					'U_OVERVIEW_PAGE'			=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')),
					'HANGMAN_DOT'				=> '&#8226;'
				));

				if (utf8_strlen($hangman_msg) != 0)
				{
					$this->template->assign_block_vars('show_help', array());
				}
				page_header($page_title);
				page_footer();
			}
			exit;
		}
		// Hangman creating
		else if ($mode == 'create')
		{
			// If not permissions to create
			if (!$this->auth->acl_get('u_hangman_create'))
			{
				meta_refresh(3, append_sid($this->root_path . 'index.' . $this->php_ext, "mode=overview"));
				trigger_error($this->user->lang['ERROR_ONLY_ADMIN_CREATE']);
			}
			include($this->root_path . 'includes/functions_posting.' . $this->php_ext);
			include($this->root_path . 'includes/functions_display.' . $this->php_ext);

			generate_smilies('inline', false);
			$this->user->add_lang('posting');

			//Template variables for the navigation
			$this->template->assign_block_vars('navlinks', array(
				'FORUM_NAME'	=> $this->user->lang['CREATION_DONE_PAGE'],
				'U_VIEW_FORUM'	=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'create')),
			));

			$this->template->set_filenames(array(
				'body' => 'hangman/hangman_create.html'
			));

			// Show rules
			if ($hangman_config['hangman_rules_enable'])
			{
				$this->template->assign_vars(array(
					'S_HANGMAN_RULES'	=> true,
				));
			}

			// Tries
			$tries_list = '';
			for ($i = $hangman_config['minimum_tries']; $i <= $hangman_config['maximum_tries']; $i++)
			{
				$tries_list .= '<option value="' . $i . '" ' . ($i == $hangman_config['standart_tries'] ? 'selected="selected"' : '') . '>' . $i . '</option>';
			}

			// Days
			$days_list = '';
			for ($i = 0; $i < 30; $i++)
			{
				$days_list .= '<option value="' . $i . '" ' . ($i == $hangman_config['standart_days'] ? 'selected="selected"' : '') . '>' . $i . '</option>';
			}

			$this->template->assign_vars(array(
				'GUELTIGE_TAGE_LIST'	=> $days_list,
				'TRIES_LIST'			=> $tries_list,
				'HANGMAN_CREATE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'new')),
				'L_CREATE_HELP_HANGMAN'	=> ($hangman_config['maximum_letters'] != -1) ? sprintf($this->user->lang['CREATE_HELP_HANGMAN'], $hangman_config['minimum_letters'], $hangman_config['maximum_letters']) : sprintf($this->user->lang['CREATE_HELP_HANGMAN_UNLIMITED'], $hangman_config['minimum_letters'], $hangman_config['maximum_letters']),
				'L_CREATE_HELP_HELP'	=> ($hangman_config['min_help'] == 0) ? '' : sprintf($this->user->lang['CREATE_HELP_HELP'], $hangman_config['min_help']),
				'L_STANDART_TITLE'		=> (isset($title)) ? $title : $hangman_config['standart_title'],
				'L_HANG_HELP'			=> $hilfe,
				'L_HANG_WORD'			=> $wort,
				'U_SMILIES'				=> append_sid("posting.$this->php_ext", "mode=smilies"),
				'U_HANGMAN'				=> $this->helper->route('dmzx_hangmangame_controller'),
				'U_HIGHSCORE_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore')),
				'U_HELP_PAGE'			=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'help')),
				'U_OVERVIEW_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')),
				'S_BBCODE_ALLOWED'		=> $this->config['allow_bbcode'] && $hangman_config['allow_bbcode'] ? true : false,
				'S_SMILIES_ALLOWED'		=> $this->config['allow_bbcode'] && $hangman_config['allow_bbcode'] && $this->config['allow_smilies'] && $hangman_config['allow_smilies'] ? true : false,
			));

			if (intval($hangman_config['maximum_letters']) > 0 && intval($hangman_config['maximum_letters']) < 255)
			{
				$this->template->assign_block_vars('switch_shorter_words', array(
					'MAXLENGTH'=> $hangman_config['maximum_letters']
				));
			}
			else
			{
				$this->template->assign_block_vars('switch_longer_words', array());
			}

			page_header($page_title);
			page_footer();
		}
		// Hangman eintragen
		else if ($mode == 'new')
		{
			if (!check_form_key('hangman_key'))
			{
				trigger_error('FORM_INVALID');
			}

			if (!$confirm)
			{
				redirect($this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')));

				exit;
			}

			if (!$this->auth->acl_get('u_hangman_create'))
			{
				meta_refresh(3, $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')));
				trigger_error($this->user->lang['ERROR_ONLY_ADMIN_CREATE']);
			}

			// Convert special chars (german,.. ;)
			if ($hangman_config['convert_special_l'])
			{
				$wort = str_replace(utf8_str_split(HANGMAN_GERMAN_LETTERS . utf8_strtolower(HANGMAN_GERMAN_LETTERS)), array('AE', 'OE', 'UE', 'ss', 'ae', 'oe', 'ue', 'ss',), $wort);
			}

			// Make uppercase ?
			if ($hangman_config['make_words_upper'])
			{
				$wort = utf8_strtoupper($wort);
			}

			// Convert space to underline
			$wort = str_replace(chr(32), '_', $wort);
			$fehlermsg = '';

			// Something missing ?
			if (!$wort)
			{
				$fehlermsg = $this->user->lang['ERROR_HANGMAN_WORD'];
			}
			else if (!$subject)
			{
				$fehlermsg = $this->user->lang['ERROR_HANGMAN_SUBJECT'];
			}
			else if (utf8_strlen($hilfe) < $hangman_config['min_help'])
			{
				$fehlermsg = $this->user->lang['ERROR_HANGMAN_HELP'];
			}
			else if (!$maximal_versuche || $maximal_versuche < $hangman_config['minimum_tries'] || $maximal_versuche > $hangman_config['maximum_tries'])
			{
				$fehlermsg = $this->user->lang['ERROR_HANGMAN_TRIES'];
			}
			else if (($gueltige_tage === null) || $gueltige_tage < 0 || $gueltige_tage > 30)
			{
				$fehlermsg = $this->user->lang['ERROR_HANGMAN_LEASE'];
			}
			// Something wrong?
			else if (!$this->functions->check_word($wort, $remaining))
			{
				$fehlermsg = sprintf($this->user->lang['ERROR_HANGMAN_FORBIDDEN_LETTERS'], $hangman_config['letters'], $remaining);
			}
			else if (utf8_strlen($wort) > $hangman_config['maximum_letters'] && $hangman_config['maximum_letters'] >= 0)
			{
				$fehlermsg = $this->user->lang['ERROR_HANGMAN_TOLONG'];
			}
			else if (utf8_strlen($wort) < $hangman_config['minimum_letters'])
			{
				$fehlermsg = sprintf($this->user->lang['ERROR_IS_SHORT'], $hangman_config['minimum_letters']);
			}

			// Parse checks successfull ;)
			if (intval($hangman_config['games_per_day']) > 0)
			{
				$today_begin = strtotime(date('Ymd', time()));

				$sql = 'SELECT COUNT(hangman_id) AS total
					FROM ' . $this->hangmangame_table_hangman_words . '
					WHERE user_id = ' . (int) $user_id . '
						AND time_created >= ' . $today_begin;
				$result = $this->db->sql_query($sql);
				$row = $this->db->sql_fetchrow($result);
				$this->db->sql_freeresult($result);

				$today_count = intval($row['total']);

				if ($today_count > intval($hangman_config['games_per_day']))
				{
					$fehlermsg = $this->user->lang['ERROR_TO_MUCH'];
				}
			}
			if (!$fehlermsg)
			{
				include_once($this->root_path . 'includes/message_parser.' . $this->php_ext);
				$message_parser = new \parse_message();

				$fehlermsg = $this->user->lang['NEW_SUCESS'];
				// Parse the stuff for postingstyle in subject and helpmessage
				$uid = $bitfield = $options = '';
				$allow_urls = true;
				$allow_bbcode	= ($this->config['allow_bbcode'] && $hangman_config['allow_bbcode']) ? true : false;
				$allow_smilies	= ($allow_bbcode && $this->config['allow_smilies'] && $hangman_config['allow_smilies']) ? true : false;
				$message = trim($subject);
				generate_text_for_storage($message, $uid, $bitfield, $options, $allow_bbcode, $allow_urls, $allow_smilies);
				$subject = $message;
				$subject_uid = $uid;
				$subject_bitfield = $bitfield;
				$subject_options = $options;
				$uid = $bitfield = $options = '';
				if ($hilfe == '')
				{
					$message_parser->message = '';
				}
				else
				{
					$message_parser->message = trim($hilfe);
					generate_text_for_storage($message_parser->message, $uid, $bitfield, $options, $allow_bbcode, $allow_urls, $allow_smilies);
				}

				// how long is time_limit 4 hangman ?
				if ($gueltige_tage != 0)
				{
					$gueltige_zeit = time() + 60 * 60 * 24 * $gueltige_tage;
				}
				else
				{
					$gueltige_zeit = 0;
				}

				$sql_ary = array(
					'user_id'				=> (int) 	$user_id,
					'hangman_word'			=> (string) $wort,
					'hangman_subject'		=> (string) $subject,
					'hangman_help'			=> (string) $message_parser->message,
					'max_tries' 			=> (int) 	$maximal_versuche,
					'bbcode_uid'			=> (string) $subject_uid,
					'bbcode_bitfield'		=> (string) $subject_bitfield,
					'bbcode_flags'			=> (int) 	$subject_options,
					'help_bbcode_uid'		=> (string) $uid,
					'help_bbcode_bitfield'	=> (string) $bitfield,
					'help_bbcode_flags'		=> (int) 	$options,
					'time_created'			=> (int) 	time(),
					'time_limit'			=> (int) 	$gueltige_zeit,
				);

				$sql = 'INSERT INTO ' . $this->hangmangame_table_hangman_words . ' ' . $this->db->sql_build_array('INSERT', $sql_ary);
				$this->db->sql_query($sql);

				// update score ;)
				$this->functions->update_score(0, 0, 1, $user_id, $user_id);
				$this->functions->counter();
			}
			else
			{
				meta_refresh(6, $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'create')));
				trigger_error($fehlermsg);
			}

			$u_back = '<a href="' . $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'create')) . '" class="nav">' . $this->user->lang['TRY_AGAIN'] . '</a>';

			$this->template->set_filenames(array(
				'body' => 'hangman/hangman_new.html'
			));

			$this->template->assign_vars(array(
				'HANGMAN_WORT'			=> $wort,
				'ERROR_MSG'				=> $fehlermsg,
				'U_HIGHSCORE_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore')),
				'U_CREATION_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'create')),
				'U_HELP_PAGE'			=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'help')),
				'U_OVERVIEW_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')),
				'U_HANGMAN'				=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')),
				'U_BACK'				=> $u_back
			));

			page_header($page_title);
			page_footer();
		}
		// Highscore
		else if ($mode == 'highscore')
		{
			$sql ='';

			if (!utf8_strlen($hfilter))
			{
				$hfilter = $hangman_config['highscore_filter'];
			}

			$sql_part = '';

			switch ($hfilter)
			{
				default:
				case 'won':
					$sql_part = 'ORDER BY won ' . utf8_strtoupper($horder);
				break;
				case 'lost':
					$sql_part = 'ORDER BY lost ' . utf8_strtoupper($horder);
				break;
				case 'score':
					$sql_part = 'ORDER BY points ' . utf8_strtoupper($horder);
				break;
				case 'created':
					$sql_part = 'ORDER BY created ' . utf8_strtoupper($horder);
				break;
				case 'r_letters':
					$sql_part = 'ORDER BY r_letters ' . utf8_strtoupper($horder);
				break;
				case 'w_letters':
					$sql_part = 'ORDER BY w_letters ' . utf8_strtoupper($horder);
				break;
			}

			$sql = 'SELECT *
				FROM ' . $this->hangmangame_table_hangman_score . '
				WHERE user_id <> ' . ANONYMOUS . '
					AND user_id IS NOT NULL
					AND user_id <> 0 ' . $sql_part;
			$result = $this->db->sql_query_limit($sql, $hlimit, $start);
			$i = 0;
			while ($row = $this->db->sql_fetchrow($result))
			{
				$r_letters	= $row['r_letters'];
				$w_letters	= $row['w_letters'];

				if ($hangman_config['hsc_letters_p'])
				{
					$insg = $r_letters + $w_letters;
					if ($insg > 0)
					{
						$r_letters = $r_letters.'<span style="color:#a4a6a3;">(' . intval( ($r_letters * 100) /( $r_letters + $w_letters)).'%)</span>';
						$w_letters = $w_letters.'<span style="color:#a4a6a3;">(' . intval( ($w_letters * 100) /( $r_letters + $w_letters)).'%)</span>';
					}
					else
					{
						$r_letters = $r_letters.'<span style="color:#a4a6a3;">(0%)</span>';
						$w_letters = $w_letters.'<span style="color:#a4a6a3;">(0%)</span>';
					}
				}

				$this->template->assign_block_vars('highscore', array(
					'POS'		=>	($start + $i + 1),
					'USERNAME'	=>	(($row['user_id'] == ANONYMOUS || $row['user_id'] == 0) && $hangman_config['hangman_bot']) ? $hangman_config['bot_name'] : $this->functions->get_username($row['user_id']),
					'WON'		=>	$row['won'],
					'LOST'		=>	$row['lost'],
					'CREATED'	=>	$row['created'],
					'POINTS'	=>	$row['points'],
					'R_LETTERS'	=>	$r_letters,
					'W_LETTERS'	=>	$w_letters,
					'KLASSE'	=>	(($i % 2) == 1) ? "row2" : "row1",
				));
				$i++;
			}
			$this->db->sql_freeresult($result);

			//Template variables for the navigation
			$this->template->assign_block_vars('navlinks', array(
				'FORUM_NAME'	=> $this->user->lang['HIGHSCORE_PAGE'],
				'U_VIEW_FORUM'	=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore')),
			));

			$this->template->set_filenames(array(
				'body' => 'hangman/hangman_highscore.html'
			));

			$sql = 'SELECT COUNT(*) AS total
				FROM ' . $this->hangmangame_table_hangman_score . '
				WHERE user_id <> ' . ANONYMOUS . '
					AND user_id IS NOT NULL
					AND user_id <> 0';
			$result = $this->db->sql_query($sql);
			$row = $this->db->sql_fetchrow($result);
			$this->db->sql_freeresult($result);

			//Limit list
			$hlimit_ary = array(10, 20, 50, 100);
			$hlimit_list = '';

			foreach ($hlimit_ary as $var)
			{
				$hlimit_list .= '<option value="' . $var . '" ' . ($var == $hlimit ? 'selected="selected"' : '') . '>' . $this->user->lang['HANGMAN_TOP'] . $var . '</option>';
			}
			//Order list
			$horder_list = '<option value="asc" ' . ($horder == 'asc' ? 'selected="selected"' : '') . '>' . $this->user->lang['HIGHSCORE_ASC'] . '</option>';
			$horder_list .= '<option value="desc" ' . ($horder == 'desc' ? 'selected="selected"' : '') . '>' . $this->user->lang['HIGHSCORE_DESC'] . '</option>';
			//Higscore filter_list
			$higscore_filters = array(
				'won'			=>	$this->user->lang['ORDER_BY_WON'],
				'lost'			=>	$this->user->lang['ORDER_BY_LOST'],
				'score'			=>	$this->user->lang['ORDER_BY_POINTS'],
				'created'		=>	$this->user->lang['ORDER_BY_CREATED'],
				'r_letters'		=>	$this->user->lang['ORDER_BY_R_LETTERS'],
				'w_letters'		=>	$this->user->lang['ORDER_BY_W_LETTERS'],
			);
			$highscore_filter_list = '';

			foreach ($higscore_filters as $key => $var)
			{
				$highscore_filter_list .= '<option value="' . $key . '" ' . ($key == $hfilter ? 'selected="selected"' : '') . '>' . $var . '</option>';
			}

			$pagination_url = $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore', 'hfilter' => $hfilter));
			//Start pagination
			$this->pagination->generate_template_pagination($pagination_url, 'pagination', 'start', $row['total'], $hlimit, $start);

			$this->template->assign_vars(array(
				'L_HIGHSCORE_ACTION'	=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore')),
				'U_ORDER_BY_WON'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore', 'order' => 'won')),
				'U_ORDER_BY_LOST'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore', 'order' => 'lost')),
				'U_ORDER_BY_CREATED'	=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore', 'order' => 'create')),
				'U_ORDER_BY_POINTS'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore', 'order' => 'points')),
				'U_ORDER_BY_R_LETTERS'	=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore', 'order' => 'r_letters')),
				'U_ORDER_BY_W_LETTERS'	=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore', 'order' => 'w_letters')),
				'U_HIGHSCORE_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore')),
				'U_CREATION_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'create')),
				'U_HELP_PAGE'			=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'help')),
				'U_OVERVIEW_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')),
				'U_HANGMAN'				=> $this->helper->route('dmzx_hangmangame_controller'),
				'U_OVERVIEW_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')),
				'HFILTER_LIST'			=> $highscore_filter_list,
				'HORDER_LIST'			=> $horder_list,
				'HLIMIT_LIST'			=> $hlimit_list,
				'TOTAL_PLAYERS'			=> $this->user->lang('TOTAL_HANGMAN_PLAYERS', (int) $row['total']),
			));
			page_header($page_title);
			page_footer();
		}
		else if ($mode == 'delete')
		{
			if (!$this->auth->acl_get('a_hangman_set') && !$this->auth->acl_get('m_hangman_delete'))
			{
				redirect($this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')));
			}
			else if ($confirm == $this->user->lang['ACP_YES'])
			{
				if (!check_form_key('hangman_key'))
				{
					trigger_error('FORM_INVALID');
				}

				$del_score = $this->request->variable('del_score', '') ? true : false;
				$this->functions->delete($game_index, $del_score);
				redirect($this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')));
			}
			else if ($confirm == $this->user->lang['ACP_NO'])
			{
				redirect($this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')));
			}

			$this->template->set_filenames(array(
				'body' => 'hangman/hangman_admin.html'
			));

			//Template variables for the navigation
			$this->template->assign_block_vars('navlinks', array(
				'FORUM_NAME'	=> $this->user->lang['HANGMAN_DELETE'],
				'U_VIEW_FORUM'	=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'delete', 'game_index' => $game_index)),
			));

			$sql = 'SELECT *
				FROM ' . $this->hangmangame_table_hangman_words . '
				WHERE hangman_id = ' . (int) $game_index;
			$result = $this->db->sql_query($sql);
			$row = $this->db->sql_fetchrow($result);
			$this->db->sql_freeresult($result);

			$hangman_subject = generate_text_for_display($row['hangman_subject'], $row['bbcode_uid'], $row['bbcode_bitfield'], $row['bbcode_flags']);
			$l_admin_text = sprintf($this->user->lang['ACP_TEXT'],'<strong>"' . $hangman_subject . '"</strong>');
			$l_admin_text .= '<br /><br /><input type="checkbox" class="liteoption" name="del_score" checked="checked" /><span class="gen">' . $this->user->lang['ACP_TEXT_SCORE'].'</span>';

			$this->template->assign_vars(array(
				'HANGMAN_SUBJECT'		=> $hangman_subject,
				'L_ADMIN_TEXT'			=> $l_admin_text,
				'L_ADMIN_TITLE'			=> $this->user->lang['ACP_TITLE'],
				'L_ADMIN_YES'			=> $this->user->lang['ACP_YES'],
				'L_ADMIN_NO'			=> $this->user->lang['ACP_NO'],
				'U_HANGMAN'				=> $this->helper->route('dmzx_hangmangame_controller'),
				'U_OVERVIEW_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')),
				'U_ADMINISTRATION'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'delete', 'game_index' =>	$game_index)),
			));

			page_header($page_title);
			page_footer();
		}

		// Multi delete
		else if ($mode == 'multi_del')
		{
			if (!$this->auth->acl_get('a_hangman_set') && !$this->auth->acl_get('m_hangman_delete'))
			{
				redirect($this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')));
				exit;
			}
			else if ($this->request->variable('submit', '') != '')
			{
				$hangman_ids = $this->request->variable('hangman_id', array(0));

				if (confirm_box(true))
				{
					$this->functions->multi_delete($hangman_ids);
					$redirect = $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview'));
					meta_refresh(3, $redirect);
					trigger_error(sprintf($this->user->lang['DELETE_MARKED_HANGMEN_SUCCESS'], sizeof($hangman_ids)) . '<br /><br />' . sprintf($this->user->lang['HANGMAN_RETURN_BACK_OVERVIEW'], '<a href="' . $redirect . '">', '</a>'));
				}
				else
				{
					$s_hidden_fields = array(
						'submit'	=> true,
						'hangman_id'	=> $hangman_ids
					);
					confirm_box(false, $this->user->lang['DELETE_MARKED_HANGMEN'], build_hidden_fields($s_hidden_fields));
				}
			}
			redirect($this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')));
			exit;
		}
		// Help
		else
		{
			$this->template->set_filenames(array(
				'body'	=> 'hangman/hangman_help.html'
			));

			//Template variables for the navigation
			$this->template->assign_block_vars('navlinks', array(
				'FORUM_NAME'	=> $this->user->lang['HELP_PAGE'],
				'U_VIEW_FORUM'	=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'help')),
			));

			$this->template->assign_vars(array(
				'L_HANGMAN'				=> $this->user->lang['HANGMAN'],
				'U_HIGHSCORE_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'highscore')),
				'U_CREATION_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'create')),
				'U_HELP_PAGE'			=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'help')),
				'U_OVERVIEW_PAGE'		=> $this->helper->route('dmzx_hangmangame_controller', array('mode' => 'overview')),
				'U_HANGMAN'				=> $this->helper->route('dmzx_hangmangame_controller'),
			));

			// Some help for overview
			$this->template->assign_block_vars('help', array(
				'L_HELP_TOPIC'			=> $this->user->lang['OVERVIEW_PAGE'],
				'L_HELP_TEXT'			=> $this->user->lang['HELP_OVERVIEW']
			));

			// Some help for creation
			$this->template->assign_block_vars('help', array(
				'L_HELP_TOPIC'			=> $this->user->lang['CREATION_PAGE'],
				'L_HELP_TEXT'			=> $this->user->lang['HELP_CREATION']
			));

			// Some help for highscore
			$this->template->assign_block_vars('help',array(
				'L_HELP_TOPIC'			=> $this->user->lang['HIGHSCORE_PAGE'],
				'L_HELP_TEXT'			=> $this->user->lang['HELP_HIGHSCORE']
			));

			if ($hangman_config['creator_bonus'])
			{
				// Some help for additional points
				$this->template->assign_block_vars('help',array(
					'L_HELP_TOPIC'			=> $this->user->lang['ADD_POINTS_PAGE'],
					'L_HELP_TEXT'			=> sprintf($this->user->lang['HELP_ADD_POINTS'], $hangman_config['creator_bonus_size']),
				));
			}

			//some help for highscore
			$this->template->assign_block_vars('help', array(
				'L_HELP_TOPIC'	=> $this->user->lang['HIGHSCORE_PAGE'],
				'L_HELP_TEXT'	=> sprintf($this->user->lang['HELP_HIGHSCORE_SYSTEM'], $hangman_config['score_create'], $hangman_config['score_won'], $hangman_config['score_letters'], $hangman_config['score_lost'])
			));

			page_header($page_title);
			page_footer();
		}
	}
}
	