<?php
/**
*
* @package phpBB Extension - Hangman Game
* @copyright (c) 2017 dmzx - http://www.dmzx-web.net
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace dmzx\hangmangame\core;

class functions_hangmangame
{
	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\user */
	protected $user;

	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\extension\manager */
	protected $extension_manager;

	/**
	* The database tables
	*
	* @var string
	*/
	protected $hangmangame_table_hangman_quess;

	protected $hangmangame_table_hangman_config;

	protected $hangmangame_table_hangman_words;

	protected $hangmangame_table_hangman_score;

	/**
	* Constructor
	*
	* @param \phpbb\template\template		 	$template
	* @param \phpbb\user						$user
	* @param \phpbb\auth\auth					$auth
	* @param \phpbb\db\driver\driver_interface	$db
	* @param \phpbb\config\config				$config
	* @param \phpbb\request\request		 		$request
	* @param \phpbb\extension\manager 			$extension_manager
	* @param string 							$hangmangame_table_hangman_quess
	* @param string 							$hangmangame_table_hangman_config
	* @param string 							$hangmangame_table_hangman_words
	* @param string 							$hangmangame_table_hangman_score
	*
	*/
	public function __construct(
		\phpbb\template\template $template,
		\phpbb\user $user,
		\phpbb\auth\auth $auth,
		\phpbb\db\driver\driver_interface $db,
		\phpbb\config\config $config,
		\phpbb\request\request $request,
		\phpbb\extension\manager $extension_manager,
		$hangmangame_table_hangman_quess,
		$hangmangame_table_hangman_config,
		$hangmangame_table_hangman_words,
		$hangmangame_table_hangman_score
	)
	{
		$this->template 						= $template;
		$this->user 							= $user;
		$this->auth 							= $auth;
		$this->db 								= $db;
		$this->config 							= $config;
		$this->request 							= $request;
		$this->extension_manager				= $extension_manager;
		$this->hangmangame_table_hangman_quess	= $hangmangame_table_hangman_quess;
		$this->hangmangame_table_hangman_config = $hangmangame_table_hangman_config;
		$this->hangmangame_table_hangman_words 	= $hangmangame_table_hangman_words;
		$this->hangmangame_table_hangman_score 	= $hangmangame_table_hangman_score;
	}

	function get_lost_time($user_id, $hangman_id)
	{
		$sql = 'SELECT *
			FROM ' . $this->hangmangame_table_hangman_quess . '
			WHERE user_id = ' . (int) $user_id . '
				AND hangman_id = ' . (int) $hangman_id;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);

		return $row['lost_time'];
	}

	/**
	* function which gets all who quessed a hangman except the user given by id
	*/
	function get_quessors($except_id, $game_index)
	{
		$quessors = array();

		$sql = 'SELECT *
			FROM ' . $this->hangmangame_table_hangman_quess . '
			WHERE hangman_id = ' . (int) $game_index . '
				AND user_id <> ' . (int) $except_id;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrowset($result);
		$quessors_counts = sizeof($row);

		for ($i = 0; $i < $quessors_counts; $i++)
		{
			$quessors[$i]['user_id'] = $row[$i]['user_id'];
			$quessors[$i]['name'] = $this->get_username($quessors[$i]['user_id']);
			$quessors[$i]['quess_count'] = $row[$i]['quess_count'];
		}
		$this->db->sql_freeresult($result);

		return $quessors;
	}

	function get_quessors_count($except_id, $game_index)
	{
		$sql = 'SELECT *
			FROM ' . $this->hangmangame_table_hangman_quess . '
			WHERE hangman_id = ' . (int) $game_index . '
				AND user_id <> ' . (int) $except_id;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrowset($result);
		$quessors_count = sizeof($row);

		$this->db->sql_freeresult($result);

		return $quessors_count;
	}

	/**
	* function will get username to given user_id
	* $no_link 	true	gives only username
	*/
	function get_username($user_id)
	{
		if ($user_id == 0)
		{
			$user_id = ANONYMOUS;
		}
		$res_name = '';
		$sql = 'SELECT username, user_colour
			FROM ' . USERS_TABLE. '
			WHERE user_id = ' . (int) $user_id;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);

		return	get_username_string('full', $user_id, $row['username'], $row['user_colour']);
 	}

	/**
	* function which get some stats ->
	* Count all, Today created, Time up, Lost, Won, Best Player
	*/
	function get_stats()
	{
		$hangman_config = $this->hangman_config();

		$stats = array(
			'count' 			=> 0,
			'today'				=> 0,
			'time_up'			=> 0,
			'won'				=> 0,
			'best_player'		=> 0,
			'total_hangmans'	=> 0,
		);

		//get online hangmans (so not quessed yet)
		$sql = 'SELECT COUNT(hangman_id) AS total
			FROM ' . $this->hangmangame_table_hangman_words . '
			WHERE bwon = 0';
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);
		$count = $row['total'];
		$stats['count'] = intval($count);

		//get total hangmans incl. quessed
		$sql = 'SELECT COUNT(hangman_id) AS total
			FROM ' . $this->hangmangame_table_hangman_words;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);
		$count = $row['total'];
		$stats['total_hangmans'] = intval($count);

		//get count today if wanted
		$today_begin = strtotime(date('Ymd', time()));
		$sql = 'SELECT COUNT(hangman_id) AS total
			FROM ' . $this->hangmangame_table_hangman_words . '
			WHERE time_created >= ' . $today_begin;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);
		$stats['today'] = $row['total'];

		//get time up
		$sql = 'SELECT COUNT(hangman_id) AS total
		FROM ' . $this->hangmangame_table_hangman_words . '
		WHERE time_limit <= ' . time() . '
			AND time_limit <> 0
			AND bwon <> 1';
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);
		$stats['time_up'] = $row['total'];

		//get online hangmans (so not quessed yet)
		$sql = 'SELECT COUNT(hangman_id) AS total
			FROM ' . $this->hangmangame_table_hangman_words . '
			WHERE bwon = 0';
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);
		$count = $row['total'];
		$stats['count'] = intval($count) - $stats['time_up'];

		//get won if wanted
		$sql = 'SELECT COUNT(hangman_id) AS total
			FROM ' . $this->hangmangame_table_hangman_words . '
			WHERE bwon = 1';
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);
		$stats['won'] = $row['total'];

		//get best player if wanted
		$sql = 'SELECT user_id
			FROM ' . $this->hangmangame_table_hangman_score . '
			WHERE user_id <> ' . ANONYMOUS . '
			ORDER BY points DESC';
		$result = $this->db->sql_query_limit($sql, 1);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);
		$stats['best_player'] = $row['user_id'];

		return $stats;
	}

	/**
	* main function to update hangman score in hanghighscore
	* and if wanted in points mod...
	*/
	function update_score($add_lost = 0, $add_won = 0, $add_created = 0, $creator_id = 0, $quessor_id = 0, $add_r_letters = 0, $add_w_letters = 0)
	{
		$hangman_config = $this->hangman_config();
		if ($creator_id == 0 && $quessor_id == 0)
		{
			return true;
		}
		$sql = 'SELECT *
			FROM ' . $this->hangmangame_table_hangman_score . '
			WHERE user_id = ' . (int) $creator_id;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);

		//if creator hasn't a score
		if (!$row && $creator_id != 0)
		{
			$sql_ary =	array(
				'user_id'		=> (int) $creator_id,
				'won'			=> 0,
				'created'		=> 0,
				'lost' 			=> 0,
				'points'		=> 0,
				'r_letters'		=> 0,
				'w_letters'		=> 0,
			);
			$sql = 'INSERT INTO ' . $this->hangmangame_table_hangman_score . ' ' . $this->db->sql_build_array('INSERT', $sql_ary);
			$this->db->sql_query($sql);
		}

		$sql = 'SELECT *
			FROM ' . $this->hangmangame_table_hangman_score . '
			WHERE user_id = ' . (int) $quessor_id;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);

		//if quessor hasn't a score
		if (!$row && $quessor_id != 0)
		{
			$sql_ary =	array(
				'user_id'		=> (int) $quessor_id,
				'won'			=> 0,
				'created'		=> 0,
				'lost' 			=> 0,
				'points'		=> 0,
				'r_letters'		=> 0,
				'w_letters'		=> 0,
			);

			$sql = 'INSERT INTO ' . $this->hangmangame_table_hangman_score . ' ' . $this->db->sql_build_array('INSERT', $sql_ary);
			$this->db->sql_query($sql);
		}

		if ($hangman_config['anonymous_points'] || $creator_id <> ANONYMOUS && $creator_id )
		{
			//update creator ;)
			$add_points = (($hangman_config['creator_bonus'] && $hangman_config['creator_bonus_size'] > 0) ? $add_lost * intval($hangman_config['creator_bonus_size']) : 0) + $add_created * intval($hangman_config['score_create']);
			$sql = 'UPDATE ' . $this->hangmangame_table_hangman_score . "
				SET points = points + $add_points
				WHERE user_id = " . $creator_id;
			$this->db->sql_query($sql);
		}

		if ($quessor_id != 0 && ($hangman_config['anonymous_points'] || $quessor_id <> ANONYMOUS ))
		{
			//update the other ;)
			$sql = 'SELECT *
				FROM ' . $this->hangmangame_table_hangman_score . '
				WHERE user_id = ' . (int) $quessor_id;
			$result = $this->db->sql_query($sql);
			$row = $this->db->sql_fetchrow($result);
			$this->db->sql_freeresult($result);

			$games_lost 	= $row['lost']	+ $add_lost;
			$games_won 		= $row['won']	+ $add_won;
			$games_created 	= $row['created'] + $add_created;
			$letters_won	= $row['r_letters'] + $add_r_letters;
			$letters_lost	= $row['w_letters'] + $add_w_letters;
			$add_points	= $row['points'] + $add_won * intval($hangman_config['score_won']) + $add_r_letters * intval($hangman_config['score_letters']);

			$sql_ary = array(
				'lost' 			=> (int) $games_lost,
				'won' 			=> (int) $games_won,
				'created' 		=> (int) $games_created,
				'points' 		=> (int) $add_points,
				'r_letters' 	=> (int) $letters_won,
				'w_letters' 	=> (int) $letters_lost,
			);

			$sql = 'UPDATE ' . $this->hangmangame_table_hangman_score . '
				SET ' . $this->db->sql_build_array('UPDATE', $sql_ary) . '
				WHERE user_id = ' . (int) $quessor_id;
			$this->db->sql_query($sql);

			//here comes some cash/points mod stuff
			if ($hangman_config['points_mod_installed'] && $this->config['points_enable'] && isset($this->config['points_name']) && $quessor_id != ANONYMOUS )
			{
				$old_points = 0;
				$sql = 'SELECT *
					FROM ' . USERS_TABLE . '
					WHERE user_id = ' . (int) $quessor_id;
				$result = $this->db->sql_query($sql);
				$points_row = $this->db->sql_fetchrow($result);
				$this->db->sql_freeresult($result);
				$old_points = $points_row['user_points'];
				$new_points = $old_points
					- ($add_lost * floatval($hangman_config['points_mod_lost']))
					+ ($add_won * floatval($hangman_config['points_mod_won']))
					+ ($add_created * floatval($hangman_config['points_mod_created']))
					+ ($add_r_letters * floatval($hangman_config['points_mod_letters']));

				$sql = 'UPDATE ' . USERS_TABLE . '
					SET user_points = ' . $new_points . '
				WHERE user_id = ' . $quessor_id;
				$this->db->sql_query($sql);

				//Update creator if guessor lost
				if ($creator_id && $creator_id <> ANONYMOUS)
				{
					$sql = 'UPDATE ' . USERS_TABLE . '
						SET user_points = user_points + ' .	($hangman_config['creator_bonus'] && $hangman_config['points_mod_creator_reward'] > 0 ? $add_lost * floatval($hangman_config['points_mod_creator_reward']) : 0) . '
					WHERE user_id = ' . $creator_id;
					$this->db->sql_query($sql);
				}
			}
		}
		return true;
	}

	/**
	* deletes a hangman and adds -1 to created games if wanted
	*/
	function delete($game_index, $del_score_creator = false)
	{
		$sql = 'DELETE FROM ' . $this->hangmangame_table_hangman_quess . '
			WHERE hangman_id = ' . (int) $game_index;
		$this->db->sql_query($sql);

		$sql = 'SELECT user_id
			FROM ' . $this->hangmangame_table_hangman_words . '
			WHERE hangman_id = ' . (int) $game_index;
		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);

		//UPDATE SCORE
		if ($del_score_creator)
		{
			$this->update_score(0, 0, -1, $row['user_id'], $row['user_id']);
		}
		else
		{
			$this->update_score(0, 0, 0, $row['user_id'], $row['user_id']);
		}

		$sql = 'DELETE FROM '	. $this->hangmangame_table_hangman_words . '
			WHERE hangman_id = ' . (int) $game_index;
		$this->db->sql_query($sql);
		return;
	}

	/**
	* Multi delete
	*/
	function multi_delete($hangman_ids)
	{
		if (!is_array($hangman_ids))
		{
			if (!$hangman_ids)
			{
				return false;
			}
			$delete_ids = array($hangman_ids);
		}

		if (!sizeof($hangman_ids))
		{
			return false;
		}
		// Get Information for later deleting
		$sql = 'SELECT hangman_id
			FROM ' . $this->hangmangame_table_hangman_words . '
			WHERE ' . $this->db->sql_in_set('hangman_id', array_map('intval', $hangman_ids));
		$result = $this->db->sql_query($sql);
		$delete_rows = array();
		$num_deleted = 0;

		while ($row = $this->db->sql_fetchrow($result))
		{
			$delete_rows[$row['hangman_id']] = 1;
		}
		$this->db->sql_freeresult($result);

		unset($hangman_ids);

		if (!sizeof($delete_rows))
		{
			return false;
		}
		// Remove hangmen
		$sql = 'DELETE FROM ' . $this->hangmangame_table_hangman_words . '
			WHERE ' . $this->db->sql_in_set('hangman_id', array_keys($delete_rows));
		$this->db->sql_query($sql);

		return true;
	}
	/**
	* Updates hangman_counter
	*/
	function counter($add = 1)
	{
		$hangman_config = $this->hangman_config();
		$stats = $this->get_stats();

		if ($stats['count'] > $hangman_config['hangman_counter'])
		{
			$hangman_config['hangman_counter'] = $stats['count'] - $add;
		}
		$sql = 'UPDATE ' . $this->hangmangame_table_hangman_config . "
			SET config_value = '" . intval($hangman_config['hangman_counter'] + $add) . "'
			WHERE config_name LIKE 'hangman_counter'";
		$this->db->sql_query($sql);

		return 1;
	}

	/**
	* checks words -> replace all good letters, if a letter remains
	* it is a bad > word check failes and gives remaining
	*/
	function check_word($word, &$remaining)
	{
		$hangman_config = $this->hangman_config();
		$temp = $word;
		$allowed_letters = utf8_str_split($hangman_config['letters']);

		for ($i = 0, $size = sizeof($allowed_letters); $i < $size; $i++)
		{
			$letter = $allowed_letters[$i];
			$temp = str_replace($letter, '', $temp);
		}

		if (utf8_strlen($temp) > 0)
		{
			$remaining = $temp;
			return false;	//word is wrong
		}
		else
		{
			return true;	//word check failes
		}
	}

	/**
	* display the quessed letters
	*/
	function quessed_letters($word, $quessed_letters)
	{
		$hangman_config = $this->hangman_config();
		$counter = 0;
		$quessed_letters = $quessed_letters . str_replace(chr(32), '_', $hangman_config['auto_replace_letters']);
		$word = utf8_str_split($word);
		$quessed_letters = utf8_str_split($quessed_letters);

		for ($i = 0; $i < count($word); $i++)
		{
			$bfound = false;
			$akt_char = ($hangman_config['make_words_upper']) ? utf8_strtoupper($word[$i]) : $word[$i];

			for ($j = 0; $j < count($quessed_letters); $j++)
			{
				$quess_char = ($hangman_config['make_words_upper']) ? utf8_strtoupper($quessed_letters[$j]) : $quessed_letters[$j];
				if ($akt_char == $quess_char)
				{
					$bfound = true;
				}
			}

			if (!$bfound)
			{
				$counter++;
			}

			//this is for getting a handle for SPACE character -> dot looks better ;)
			if ($akt_char == '_' || $akt_char == chr(32))
			{
				$akt_char = '&#8226;';
			}

			$this->template->assign_block_vars('hangman_letters', array(
				'HANGMAN_CHAR' => ($bfound == true) ? $akt_char : '_'
			));
		}
		return $counter;
	}

	/**
	* adding hangmans
	*/
	function add($hang, $bignore_doubles, $max_import, $importer_id, $min_letters, $ignore_dleft, &$success)
	{
		$hangman_config = $this->hangman_config();

		// disable sql error handling
		$this->db->sql_return_on_error(true);
		$error_msg = '';

		//get the array of actual hangman words...
		$ignore_words = array();

		if ($bignore_doubles)
		{
			$sql = 'SELECT *
				FROM ' . $this->hangmangame_table_hangman_words;
			$result = $this->db->sql_query($sql);
			$rows = $this->db->sql_fetchrowset($result);
			$this->db->sql_freeresult($result);

			$crows = sizeof($rows);

			for ($x = 0; $x < $crows; $x++)
			{
				$ignore_words[] = $rows[$x]['hangman_word'];
			}
		}
		$success = 0;

		for ($i = 0, $size = count($hang); $i < $size; $i++)
		{
			$zeit = time();
			$gueltige_zeit = 0;

			if (!$ignore_dleft)
			{
				$gueltige_zeit = ($hang[$i]->days > 3) ? ($zeit + 60 * 60 * 24 * ($hang[$i]->days)) : 0;
			}

			$wort				= $hang[$i]->word;
			$help				= $hang[$i]->help;
			$hilfe				= $help;
			$title				= $hang[$i]->title;
			$subject			= $title;
			$maximal_versuche 	= $hang[$i]->tries;
			$skip				= false;
			$skip_err			= sprintf($this->user->lang['ACP_HANGMAN_SKIP'], $wort);

			if (utf8_strlen($wort) < $min_letters)
			{
				$skip_err = $this->user->lang['ACP_HANGMAN_SKIP_FEW'];
				$skip = true;
			}

			if (in_array($wort, $ignore_words))
			{
				$skip_err .= $this->user->lang['ACP_HANGMAN_SKIP_ALREADY'];
				$skip = true;
			}

			if ($hangman_config['make_words_upper'])
			{
				$wort = utf8_strtoupper($wort);
			}

			if (!$this->check_word($wort, $remaining))
			{
				$skip_err .= sprintf($this->user->lang['ACP_HANGMAN_SKIP_PARSING'], $remaining);
				$skip = true;
			}

			if (stristr($hilfe, '[/') || stristr($subject, '[/'))
			{
				$skip_err .= sprintf($this->user->lang['ACP_HANGMAN_SKIP_BBCODED']);
				$skip = true;
			}

			if ($skip === false)
			{
				$sql_ary =	array(
					'user_id'				=>	(int) $importer_id,
					'hangman_word'			=>	(string) $wort,
					'hangman_help'			=>	(string) $hilfe,
					'hangman_subject' 		=> 	(string) $subject,
					'max_tries'				=>	(int) $maximal_versuche,
					'bbcode_uid'			=>	'',
					'bbcode_bitfield'		=>	'',
					'bbcode_flags'			=>	0,
					'help_bbcode_uid'		=>	'',
					'help_bbcode_bitfield'	=>	'',
					'help_bbcode_flags'		=>	0,
					'time_created'			=>	(int) $zeit,
					'time_limit'			=>	(int) $gueltige_zeit,
				);
				$sql = 'INSERT INTO ' . $this->hangmangame_table_hangman_words . ' ' . $this->db->sql_build_array('INSERT', $sql_ary);

				if (!$this->db->sql_query($sql))
				{
					$error_msg = $error_msg . '<br />' . sprintf($this->user->lang['ACP_HANGMAN_IMPORT_SQLERR'], ($i + 1));
				}
				else
				{
					$success++;
					$this->update_score(0, 0, 1, $importer_id, $importer_id);
					$this->counter();
				}
			}
			else
			{
				$error_msg .= '<br />' . sprintf($this->user->lang['ACP_HANGMAN_SKIPPED_ERROR'], $wort, $skip_err);
			}
			if ($success >= $max_import)
			{
				$error_msg .= '<br />' . sprintf($this->user->lang['ACP_HANGMAN_IMPORT_STOP'], $success);
				break;
			}
		}
		$error_msg .= '<br />' . sprintf($this->user->lang['ACP_HANGMAN_IMPORT_SUCCESS'], $success);
		return $error_msg;
	}

	/**
	* sets and/or gets the configs for hangman
	*/
	function hangman_get_config(&$hangman_config)
	{
		$hangman_config = $this->hangman_config();
		$sql = 'SELECT *
			FROM ' . $this->hangmangame_table_hangman_config;
		$result = $this->db->sql_query($sql);
		$rows		= $this->db->sql_fetchrowset($result);
		$rowcount	= sizeof($rows);
		$this->db->sql_freeresult($result);

		if ($rowcount != count($hangman_config))
		{
			$names_ary = array();

			for ($i = 0; $i < $rowcount; $i++)
			{
				$names_ary[] = $rows[$i]['config_name'];
			}

			foreach ($hangman_config as $name => $value) //Check standard names
			{
				if (!in_array($name, $names_ary))
				{
					$sql = 'INSERT INTO ' . $this->hangmangame_table_hangman_config . " (config_name, config_value) VALUES('$name', '" . $this->db->sql_escape($value) . "')";
					$this->db->sql_query($sql);
				}
			}

			foreach ($names_ary as $key => $var) //Check acting names
			{
				if (!isset($hangman_config[$var]))
				{
					$sql = 'DELETE FROM ' . $this->hangmangame_table_hangman_config . " WHERE config_name LIKE '$var'";
					$this->db->sql_query($sql);
				}
			}

			$hangman_config = array(); //Grab new acting hangman config
			$sql = 'SELECT *
				FROM ' . $this->hangmangame_table_hangman_config;
			$result = $this->db->sql_query($sql);

			while ($row = $this->db->sql_fetchrow($result))
			{
				$hangman_config[$row['config_name']] = $row['config_value'];
			}
			$this->db->sql_freeresult($result);
		}
		else
		{
			$hangman_config = array();

			for ($i = 0; $i < $rowcount; $i++)
			{
				$hangman_config[$rows[$i]['config_name']] = $rows[$i]['config_value'];
			}
		}
		return true;
	}

	/**
	* Assign authors
	*/
	public function assign_authors()
	{
		$md_manager = $this->extension_manager->create_extension_metadata_manager('dmzx/hangmangame', $this->template);
		$meta = $md_manager->get_metadata();
		$author_names = array();
		$author_homepages = array();

		foreach (array_slice($meta['authors'], 0, 1) as $author)
		{
			$author_names[] = $author['name'];
			$author_homepages[] = sprintf('<a href="%1$s" title="%2$s">%2$s</a>', $author['homepage'], $author['name']);
		}

		$this->template->assign_vars(array(
			'HANGMANGAME_DISPLAY_NAME'		=> $meta['extra']['display-name'],
			'HANGMANGAME_AUTHOR_NAMES'		=> implode(' &amp; ', $author_names),
			'HANGMANGAME_AUTHOR_HOMEPAGES'	=> implode(' &amp; ', $author_homepages),
		));

		return;
	}

	function hangman_config()
	{
		$sql = 'SELECT config_name, config_value
			FROM ' . $this->hangmangame_table_hangman_config;
		$result = $this->db->sql_query($sql);

		while ($row = $this->db->sql_fetchrow($result))
		{
			$hangman_config[$row['config_name']] = $row['config_value'];
		}
		$this->db->sql_freeresult($result);

		return $hangman_config;
	}
}
