<?php
/**
*
* @package phpBB Extension - Hangman Game
* @copyright (c) 2017 dmzx - http://www.dmzx-web.net
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/
namespace dmzx\hangmangame\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
* Event listener
*/
class listener implements EventSubscriberInterface
{
	/** @var \phpbb\user */
	protected $user;

	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\controller\helper */
	protected $helper;

	/**
	* Constructor
	*
	* @param \phpbb\user						$user
	* @param \phpbb\template\template			$template
	* @param \phpbb\config\config				$config
	* @param \phpbb\controller\helper			$helper
	*
	*/
	public function __construct(
		\phpbb\user $user,
		\phpbb\template\template $template,
		\phpbb\config\config $config,
		\phpbb\controller\helper $helper
	)
	{
		$this->user					= $user;
		$this->template				= $template;
		$this->config				= $config;
		$this->helper 				= $helper;
	}

	static public function getSubscribedEvents()
	{
		return array(
			'core.user_setup'		=> 'load_language_on_setup',
			'core.page_header'		=> 'page_header',
			'core.permissions'		=> 'permissions',
		);
	}

	public function load_language_on_setup($event)
	{
		$lang_set_ext = $event['lang_set_ext'];
		$lang_set_ext[] = array(
			'ext_name' => 'dmzx/hangmangame',
			'lang_set' => 'common',
		);
		$event['lang_set_ext'] = $lang_set_ext;
	}

	public function page_header($event)
	{
		$this->template->assign_vars(array(
			'U_HANGMANGAME'		=> $this->helper->route('dmzx_hangmangame_controller'),
		));
	}

	// Show permissions
	public function permissions($event)
	{
		$permissions = $event['permissions'];
		$permissions += array(
			'a_hangman_set'		=> array(
				'lang'		=> 'ACL_A_HANGMAN_SET',
				'cat'		=> 'hangmangame'
			),
			'a_hangman_import'	=> array(
				'lang'		=> 'ACL_A_HANGMAN_IMPORT',
				'cat'		=> 'hangmangame'
			),
			'm_hangman_delete'	=> array(
				'lang'		=> 'ACL_M_HANGMAN_DELETE',
				'cat'		=> 'hangmangame'
			),
			'u_hangman_play'	=> array(
				'lang'		=> 'ACL_U_HANGMAN_PLAY',
				'cat'		=> 'hangmangame'
			),
			'u_hangman_create'	=> array(
				'lang'		=> 'ACL_U_HANGMAN_CREATE',
				'cat'		=> 'hangmangame'
			),
		);
		$event['permissions'] = $permissions;
		$categories['hangmangame'] = 'HANGMAN_TITLE';
		$event['categories'] = array_merge($event['categories'], $categories);
	}
}
