<?php
/**
*
* @package phpBB Extension - Hangman Game
* @copyright (c) 2017 dmzx - http://www.dmzx-web.net
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace dmzx\hangmangame\migrations;

class hangmangame_data extends \phpbb\db\migration\migration
{
	var $ext_version = '1.0.0';

	static public function depends_on()
	{
		return array(
			'\dmzx\hangmangame\migrations\hangmangame_schema',
		);
	}

	public function update_data()
	{
		return array(
			// Add config
			array('config.add', array('hangmangame_version', $this->ext_version)),
			array('config.add', array('hangman_enable', '1', true)),
			// Add permission
			array('permission.add', array('a_hangman_set', true)),
			array('permission.add', array('a_hangman_import', true)),
			array('permission.add', array('m_hangman_delete', true)),
			array('permission.add', array('u_hangman_play', true)),
			array('permission.add', array('u_hangman_create', true)),
			// Set permission
			array('permission.permission_set', array('ROLE_ADMIN_STANDARD', 'a_hangman_set')),
			array('permission.permission_set', array('ROLE_ADMIN_STANDARD', 'a_hangman_import')),
			array('permission.permission_set', array('ROLE_MOD_STANDARD', 'm_hangman_delete')),
			array('permission.permission_set', array('REGISTERED', 'u_hangman_play', 'group')),
			array('permission.permission_set', array('REGISTERED', 'u_hangman_create', 'group')),
			// Insert config data
			array('custom', array(
				array(&$this, 'insert_config_data')
			)),
		);
	}

	public function insert_config_data()
	{
		if ($this->db_tools->sql_table_exists($this->table_prefix . 'hangman_config'))
		{
			// Define sample rule data
			$config_data = array(
				array(
					'config_name' 	=> 'allow_bbcode',
						'config_value'	=> true,
					),
				array(
					'config_name' 	=> 'allow_smilies',
					'config_value'	=> true,
				),
				array(
					'config_name' 	=> 'games_per_page',
					'config_value'	=> '20',
				),
				array(
					'config_name' 	=> 'points_mod_installed',
					'config_value'	=> false,
				),
				array(
					'config_name' 	=> 'points_mod_won',
					'config_value'	=> '100',
				),
				array(
					'config_name' 	=> 'points_mod_lost',
					'config_value'	=> '10',
				),
				array(
					'config_name' 	=> 'points_mod_creator_reward',
					'config_value'	=> '5',
				),
				array(
					'config_name' 	=> 'points_mod_created',
					'config_value'	=> '50',
				),
				array(
					'config_name' 	=> 'points_mod_letters',
					'config_value'	=> '10',
				),
				array(
					'config_name' 	=> 'time_play_again',
					'config_value'	=> '30',
				),
				array(
					'config_name' 	=> 'make_words_upper',
					'config_value'	=> true,
				),
				array(
					'config_name' 	=> 'standart_filter',
					'config_value'	=> 'none',
				),
				array(
					'config_name' 	=> 'standart_days',
					'config_value'	=> '3',
				),
				array(
					'config_name' 	=> 'standart_tries',
					'config_value'	=> '2',
				),
				array(
					'config_name' 	=> 'score_lost',
					'config_value'	=> '1',
				),
				array(
					'config_name' 	=> 'score_won',
					'config_value'	=> '15',
				),
				array(
					'config_name' 	=> 'score_create',
					'config_value'	=> '1',
				),
				array(
					'config_name' 	=> 'score_letters',
					'config_value'	=> '1',
				),
				array(
					'config_name' 	=> 'hsc_letters_p',
					'config_value'	=> '1',
				),
				array(
					'config_name' 	=> 'highscore_filter',
					'config_value'	=> 'score',
				),
				array(
					'config_name' 	=> 'letters',
					'config_value'	=> 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789, .:_-?;*+!',
				),
				array(
					'config_name' 	=> 'auto_replace_letters',
					'config_value'	=> ',.- _;*+?!:',
				),
				array(
					'config_name' 	=> 'minimum_tries',
					'config_value'	=> '3',
				),
				array(
					'config_name' 	=> 'maximum_tries',
					'config_value'	=> '9',
				),
				array(
					'config_name' 	=> 'minimum_letters',
					'config_value'	=> '3',
				),
				array(
					'config_name' 	=> 'maximum_letters',
					'config_value'	=> '35',
				),
				array(
					'config_name' 	=> 'convert_special_l',
					'config_value'	=> false,
				),
				array(
					'config_name' 	=> 'show_stats',
					'config_value'	=> true,
				),
				array(
					'config_name' 	=> 'games_per_day',
					'config_value'	=> '100',
				),
				array(
					'config_name' 	=> 'requessing_lose',
					'config_value'	=> true,
				),
				array(
					'config_name' 	=> 'show_numbers',
					'config_value'	=> true,
				),
				array(
					'config_name' 	=> 'min_help',
					'config_value'	=> '0',
				),
				array(
					'config_name' 	=> 'standart_title',
					'config_value'	=> 'Hangman',
				),
				array(
					'config_name' 	=> 'hangman_counter',
					'config_value'	=> '0',
				),
				array(
					'config_name' 	=> 'hangman_version',
					'config_value'	=> $this->ext_version,
				),
				array(
					'config_name' 	=> 'hangman_rules_enable',
					'config_value'	=> '0',
				),
				array(
					'config_name' 	=> 'creator_bonus',
					'config_value'	=> true,
				),
				array(
					'config_name' 	=> 'creator_bonus_size',
					'config_value'	=> '5',
				),
				array(
					'config_name' 	=> 'anonymous_points',
					'config_value'	=> true,
				),
				array(
					'config_name' 	=> 'hangman_bot',
					'config_value'	=> false,
				),
				array(
					'config_name' 	=> 'bot_name',
					'config_value'	=> 'Hangman Bot',
				),
				array(
					'config_name' 	=> 'import_bonus',
					'config_value'	=> true,
				),
				array(
					'config_name' 	=> 'import_bonus_size',
					'config_value'	=> '1',
				),
			);
		}
		// Insert sample data
		$this->db->sql_multi_insert($this->table_prefix . 'hangman_config', $config_data);
	}
}
